@@ -140,6 +140,7 @@ class _AudioTextDALIDataset(Iterator):
         global_rank (int): Worker rank, used for partitioning shards. Defaults to 0.
         world_size (int): Total number of processes, used for partitioning shards. Defaults to 1.
         preprocessor_cfg (DictConfig): Preprocessor configuration. Supports AudioToMelSpectrogramPreprocessor and AudioToMFCCPreprocessor.
+        return_sample_id (bool): whether to return the sample_id as a part of each sample (not supported yet).
     """
 
     def __init__(
@@ -162,8 +163,15 @@ class _AudioTextDALIDataset(Iterator):
         global_rank: int = 0,
         world_size: int = 1,
         preprocessor_cfg: DictConfig = None,
+        return_sample_id: bool = False,
     ):
         self.drop_last = drop_last  # used by lr_scheduler
+        if return_sample_id:
+            raise ValueError(
+                "Currently DALI data layers don't support returning the sample_id and return_sample_id can not be enabled."
+            )
+        self.return_sample_id = return_sample_id
+
         if not HAVE_DALI:
             raise ModuleNotFoundError(
                 f"{self} requires NVIDIA DALI to be installed. "
@@ -519,6 +527,7 @@ class AudioToCharDALIDataset(_AudioTextDALIDataset):
         global_rank (int): Worker rank, used for partitioning shards. Defaults to 0.
         world_size (int): Total number of processes, used for partitioning shards. Defaults to 1.
         preprocessor_cfg (DictConfig): Preprocessor configuration. Supports AudioToMelSpectrogramPreprocessor and AudioToMFCCPreprocessor.
+        return_sample_id (bool): whether to return the sample_id as a part of each sample (not supported yet).
     """
 
     def __init__(
@@ -545,6 +554,7 @@ class AudioToCharDALIDataset(_AudioTextDALIDataset):
         global_rank: int = 0,
         world_size: int = 1,
         preprocessor_cfg: DictConfig = None,
+        return_sample_id: bool = False,
     ):
         self.labels = labels
 
@@ -571,6 +581,7 @@ class AudioToCharDALIDataset(_AudioTextDALIDataset):
             global_rank=global_rank,
             world_size=world_size,
             preprocessor_cfg=preprocessor_cfg,
+            return_sample_id=return_sample_id,
         )
 
 
@@ -607,6 +618,7 @@ class AudioToBPEDALIDataset(_AudioTextDALIDataset):
         preprocessor_cfg (DictConfig): Preprocessor configuration. Supports AudioToMelSpectrogramPreprocessor and AudioToMFCCPreprocessor.
         use_start_end_token (bool): Boolean which dictates whether to add [BOS] and [EOS] tokens to beginning and
             ending of speech respectively.
+        return_sample_id (bool): whether to return the sample_id as a part of each sample (not supported yet).
     """
 
     def __init__(
@@ -627,7 +639,9 @@ class AudioToBPEDALIDataset(_AudioTextDALIDataset):
         world_size: int = 1,
         preprocessor_cfg: DictConfig = None,
         use_start_end_token: bool = True,
+        return_sample_id: bool = False,
     ):
+
         if use_start_end_token and hasattr(tokenizer, 'bos_token'):
             bos_id = tokenizer.bos_id
         else:
@@ -670,4 +684,5 @@ class AudioToBPEDALIDataset(_AudioTextDALIDataset):
             global_rank=global_rank,
             world_size=world_size,
             preprocessor_cfg=preprocessor_cfg,
+            return_sample_id=return_sample_id,
         )
