@@ -14,12 +14,10 @@
 
 import copy
 import itertools
-import json
-import os
-import pickle as pkl
 from typing import Dict, List, Optional, Union
 
 import librosa
+import numpy as np
 import torch
 from omegaconf import DictConfig
 from omegaconf.omegaconf import open_dict
@@ -33,7 +31,6 @@ from nemo.collections.asr.losses.angularloss import AngularSoftmaxLoss
 from nemo.collections.asr.models.asr_model import ExportableEncDecModel
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
 from nemo.collections.asr.parts.preprocessing.perturb import process_augmentations
-from nemo.collections.asr.parts.utils.speaker_utils import embedding_normalize
 from nemo.collections.common.losses import CrossEntropyLoss as CELoss
 from nemo.collections.common.metrics import TopKClassificationAccuracy
 from nemo.collections.common.parts.preprocessing.collections import ASRSpeechLabel
@@ -42,17 +39,18 @@ from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.core.neural_types import *
 from nemo.utils import logging
 
-__all__ = ['EncDecSpeakerLabelModel', 'ExtractSpeakerEmbeddingsModel']
+__all__ = ['EncDecSpeakerLabelModel']
 
 
 class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
-    """Encoder decoder class for speaker label models.
+    """
+    Encoder decoder class for speaker label models.
     Model class creates training, validation methods for setting up data
     performing model forward pass.
     Expects config dict for
-    * preprocessor
-    * Jasper/Quartznet Encoder
-    * Speaker Decoder
+        * preprocessor
+        * Jasper/Quartznet Encoder
+        * Speaker Decoder
     """
 
     @classmethod
@@ -63,12 +61,6 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             List of available pre-trained models.
         """
         result = []
-        model = PretrainedModelInfo(
-            pretrained_model_name="speakerrecognition_speakernet",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/speakerrecognition_speakernet/versions/1.0.0rc1/files/speakerrecognition_speakernet.nemo",
-            description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:speakerrecognition_speakernet, NOTE: this model would be removed for next release and use only single speakernet and ecapa_tdnn models",
-        )
-        result.append(model)
 
         model = PretrainedModelInfo(
             pretrained_model_name="speakerverification_speakernet",
@@ -78,16 +70,16 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         result.append(model)
 
         model = PretrainedModelInfo(
-            pretrained_model_name="speakerdiarization_speakernet",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/speakerdiarization_speakernet/versions/1.0.0rc1/files/speakerdiarization_speakernet.nemo",
-            description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:speakerdiarization_speakernet, NOTE: this model would be removed for next release and use only single speakernet and ecapa_tdnn model",
+            pretrained_model_name="ecapa_tdnn",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/ecapa_tdnn/versions/v1/files/ecapa_tdnn.nemo",
+            description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:ecapa_tdnn",
         )
         result.append(model)
 
         model = PretrainedModelInfo(
-            pretrained_model_name="ecapa_tdnn",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/ecapa_tdnn/versions/v1/files/ecapa_tdnn.nemo",
-            description="For details about this model, please visit https://ngc.nvidia.com/catalog/models/nvidia:nemo:ecapa_tdnn",
+            pretrained_model_name="titanet_large",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/titanet_large/versions/v0/files/titanet-l.nemo",
+            description="For details about this model, please visit https://catalog.ngc.nvidia.com/orgs/nvidia/teams/nemo/models/titanet_large",
         )
         result.append(model)
 
@@ -104,12 +96,12 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         self.encoder = EncDecSpeakerLabelModel.from_config_dict(cfg.encoder)
         self.decoder = EncDecSpeakerLabelModel.from_config_dict(cfg.decoder)
         if 'angular' in cfg.decoder and cfg.decoder['angular']:
-            logging.info("Training with Angular Softmax Loss")
+            logging.info("loss is Angular Softmax")
             scale = cfg.loss.scale
             margin = cfg.loss.margin
             self.loss = AngularSoftmaxLoss(scale=scale, margin=margin)
         else:
-            logging.info("Training with Softmax-CrossEntropy loss")
+            logging.info("loss is Softmax-CrossEntropy")
             self.loss = CELoss()
         self.task = None
         self._accuracy = TopKClassificationAccuracy(top_k=[1])
@@ -129,10 +121,10 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
                 manifests_files=manifest_filepath,
                 min_duration=data_layer_config.get("min_duration", None),
                 max_duration=data_layer_config.get("max_duration", None),
-                index_by_file_id=True,  # Must set this so the manifest lines can be indexed by file ID
+                index_by_file_id=False,
             )
             labels.update(collection.uniq_labels)
-        labels = list(labels)
+        labels = list(sorted(labels))
         logging.warning(f"Total number of {len(labels)} found in all the manifest files.")
         return labels
 
@@ -177,8 +169,6 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
                 max_duration=config.get('max_duration', None),
                 min_duration=config.get('min_duration', None),
                 trim=config.get('trim_silence', False),
-                time_length=config.get('time_length', 8),
-                shift_length=config.get('shift_length', 0.75),
                 normalize_audio=config.get('normalize_audio', False),
             )
 
@@ -188,14 +178,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             collate_ds = dataset
 
         # self.labels = collate_ds.labels
-
-        if self.task == 'diarization':
-            logging.info("Setting up diarization parameters")
-            collate_fn = collate_ds.sliced_seq_collate_fn
-            shuffle = False
-        else:
-            logging.info("Setting up identification parameters")
-            collate_fn = collate_ds.fixed_seq_collate_fn
+        collate_fn = collate_ds.fixed_seq_collate_fn
 
         batch_size = config['batch_size']
         return torch.utils.data.DataLoader(
@@ -213,25 +196,16 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         train_data_layer_config['labels'] = self.labels
         if 'shuffle' not in train_data_layer_config:
             train_data_layer_config['shuffle'] = True
-        self.task = 'identification'
         self._train_dl = self.__setup_dataloader_from_config(config=train_data_layer_config)
 
     def setup_validation_data(self, val_data_layer_config: Optional[Union[DictConfig, Dict]]):
         val_data_layer_config['labels'] = self.labels
-        self.task = 'identification'
         self._validation_dl = self.__setup_dataloader_from_config(config=val_data_layer_config)
 
     def setup_test_data(self, test_data_layer_params: Optional[Union[DictConfig, Dict]]):
         if hasattr(self, 'dataset'):
             test_data_layer_params['labels'] = self.labels
 
-        if 'task' in test_data_layer_params and test_data_layer_params['task']:
-            self.task = test_data_layer_params['task'].lower()
-            self.time_length = test_data_layer_params.get('time_length', 1.5)
-            self.shift_length = test_data_layer_params.get('shift_length', 0.75)
-        else:
-            self.task = 'identification'
-
         self.embedding_dir = test_data_layer_params.get('embedding_dir', './')
         self._test_dl = self.__setup_dataloader_from_config(config=test_data_layer_params)
         self.test_manifest = test_data_layer_params.get('manifest_filepath', None)
@@ -358,11 +332,13 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         setup_finetune_model method sets up training data, validation data and test data with new
         provided config, this checks for the previous labels set up during training from scratch, if None,
         it sets up labels for provided finetune data from manifest files
+
         Args:
-        model_config: cfg which has train_ds, optional validation_ds, optional test_ds and
-        mandatory encoder and decoder model params
-        make sure you set num_classes correctly for finetune data
-        Returns: None
+            model_config: cfg which has train_ds, optional validation_ds, optional test_ds, 
+            mandatory encoder and decoder model params. Make sure you set num_classes correctly for finetune data.
+
+        Returns: 
+            None
         """
         logging.info("Setting up data loaders with manifests provided from model_config")
 
@@ -408,12 +384,22 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
 
     @torch.no_grad()
     def get_embedding(self, path2audio_file):
+        """
+        Returns the speaker embeddings for a provided audio file.
+
+        Args:
+            path2audio_file: path to audio wav file
+
+        Returns:
+            embs: speaker embeddings 
+        """
         audio, sr = librosa.load(path2audio_file, sr=None)
         target_sr = self._cfg.train_ds.get('sample_rate', 16000)
         if sr != target_sr:
             audio = librosa.core.resample(audio, sr, target_sr)
         audio_length = audio.shape[0]
         device = self.device
+        audio = np.array(audio)
         audio_signal, audio_signal_len = (
             torch.tensor([audio], device=device),
             torch.tensor([audio_length], device=device),
@@ -429,50 +415,31 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         del audio_signal, audio_signal_len
         return embs
 
+    @torch.no_grad()
+    def verify_speakers(self, path2audio_file1, path2audio_file2, threshold=0.7):
+        """
+        Verify if two audio files are from the same speaker or not.
 
-class ExtractSpeakerEmbeddingsModel(EncDecSpeakerLabelModel):
-    """
-    This Model class facilitates extraction of speaker embeddings from a pretrained model.
-    Respective embedding file is saved in self.embedding dir passed through cfg
-    """
+        Args:
+            path2audio_file1: path to audio wav file of speaker 1  
+            path2audio_file2: path to audio wav file of speaker 2 
+            threshold: cosine similarity score used as a threshold to distinguish two embeddings (default = 0.7)
 
-    def __init__(self, cfg: DictConfig, trainer: Trainer = None):
-        super().__init__(cfg=cfg, trainer=trainer)
-
-    def test_step(self, batch, batch_ix):
-        audio_signal, audio_signal_len, labels, slices = batch
-        _, embs = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
-        return {'embs': embs, 'labels': labels, 'slices': slices}
-
-    def test_epoch_end(self, outputs):
-        embs = torch.cat([x['embs'] for x in outputs])
-        slices = torch.cat([x['slices'] for x in outputs])
-        emb_shape = embs.shape[-1]
-        embs = embs.view(-1, emb_shape).cpu().numpy()
-        embs = embedding_normalize(embs)
-        out_embeddings = {}
-        start_idx = 0
-        with open(self.test_manifest, 'r') as manifest:
-            for idx, line in enumerate(manifest.readlines()):
-                line = line.strip()
-                dic = json.loads(line)
-                structure = dic['audio_filepath'].split('/')[-3:]
-                uniq_name = '@'.join(structure)
-                if uniq_name in out_embeddings:
-                    raise KeyError("Embeddings for label {} already present in emb dictionary".format(uniq_name))
-                num_slices = slices[idx]
-                end_idx = start_idx + num_slices
-                out_embeddings[uniq_name] = embs[start_idx:end_idx].mean(axis=0)
-                start_idx = end_idx
-
-        embedding_dir = os.path.join(self.embedding_dir, 'embeddings')
-        if not os.path.exists(embedding_dir):
-            os.mkdir(embedding_dir)
-
-        prefix = self.test_manifest.split('/')[-1].split('.')[-2]
-
-        name = os.path.join(embedding_dir, prefix)
-        pkl.dump(out_embeddings, open(name + '_embeddings.pkl', 'wb'))
-        logging.info("Saved embedding files to {}".format(embedding_dir))
-
-        return {}
+        Returns:  
+            True if both audio files are from same speaker, False otherwise
+        """
+        embs1 = self.get_embedding(path2audio_file1).squeeze()
+        embs2 = self.get_embedding(path2audio_file2).squeeze()
+        # Length Normalize
+        X = embs1 / torch.linalg.norm(embs1)
+        Y = embs1 / torch.linalg.norm(embs2)
+        # Score
+        similarity_score = torch.dot(X, Y) / ((torch.dot(X, X) * torch.dot(Y, Y)) ** 0.5)
+        similarity_score = (similarity_score + 1) / 2
+        # Decision
+        if similarity_score >= threshold:
+            logging.info(" two audio files are from same speaker")
+            return True
+        else:
+            logging.info(" two audio files are from different speakers")
+            return False
