@@ -112,32 +112,25 @@ class RNNTDecoder(rnnt_abstract.AbstractRNNTDecoder, Exportable):
             "states": [NeuralType((('D', 'B', 'D')), ElementType(), optional=True)],  # must always be last
         }
 
-    def _prepare_for_export(self, **kwargs):
-        self.freeze()
-
-    def input_example(self):
+    def input_example(self, max_batch=1, max_dim=1):
         """
         Generates input examples for tracing etc.
         Returns:
             A tuple of input examples.
         """
-        length = 1
-        targets = torch.full(fill_value=self.blank_idx, size=(1, length), dtype=torch.int32).to(
+        length = max_dim
+        targets = torch.full(fill_value=self.blank_idx, size=(max_batch, length), dtype=torch.int32).to(
+            next(self.parameters()).device
+        )
+        target_length = torch.randint(0, length, size=(max_batch,), dtype=torch.int32).to(
             next(self.parameters()).device
         )
-        target_length = torch.randint(0, length, size=(1,), dtype=torch.int32).to(next(self.parameters()).device)
         states = tuple(self.initialize_state(targets.float()))
         return (targets, target_length, states)
 
-    @property
-    def disabled_deployment_input_names(self):
-        """Implement this method to return a set of input names disabled for export"""
-        return set([])
-
-    @property
-    def disabled_deployment_output_names(self):
-        """Implement this method to return a set of output names disabled for export"""
-        return set([])
+    def _prepare_for_export(self, **kwargs):
+        self._rnnt_export = True
+        super()._prepare_for_export(**kwargs)
 
     def __init__(
         self,
@@ -174,8 +167,6 @@ class RNNTDecoder(rnnt_abstract.AbstractRNNTDecoder, Exportable):
             hidden_hidden_bias_scale=hidden_hidden_bias_scale,
             dropout=dropout,
         )
-
-        # Flag needed for RNNT export support
         self._rnnt_export = False
 
     @typecheck()
@@ -545,13 +536,75 @@ class RNNTDecoder(rnnt_abstract.AbstractRNNTDecoder, Exportable):
             (tuple): decoder states for given id
                 ([L x (1, H)], [L x (1, H)])
         """
+        if batch_states is not None:
+            state_list = []
+            for state_id in range(len(batch_states)):
+                states = [batch_states[state_id][layer][idx] for layer in range(self.pred_rnn_layers)]
+                state_list.append(states)
+
+            return state_list
+        else:
+            return None
+
+    def batch_concat_states(self, batch_states: List[List[torch.Tensor]]) -> List[torch.Tensor]:
+        """Concatenate a batch of decoder state to a packed state.
+
+        Args:
+            batch_states (list): batch of decoder states
+                B x ([L x (H)], [L x (H)])
+
+        Returns:
+            (tuple): decoder states
+                (L x B x H, L x B x H)
+        """
         state_list = []
-        for state_id in range(len(batch_states)):
-            states = [batch_states[state_id][layer][idx] for layer in range(self.pred_rnn_layers)]
-            state_list.append(states)
+
+        for state_id in range(len(batch_states[0])):
+            batch_list = []
+            for sample_id in range(len(batch_states)):
+                tensor = torch.stack(batch_states[sample_id][state_id])  # [L, H]
+                tensor = tensor.unsqueeze(0)  # [1, L, H]
+                batch_list.append(tensor)
+
+            state_tensor = torch.cat(batch_list, 0)  # [B, L, H]
+            state_tensor = state_tensor.transpose(1, 0)  # [L, B, H]
+            state_list.append(state_tensor)
 
         return state_list
 
+    def batch_copy_states(
+        self,
+        old_states: List[torch.Tensor],
+        new_states: List[torch.Tensor],
+        ids: List[int],
+        value: Optional[float] = None,
+    ) -> List[torch.Tensor]:
+        """Copy states from new state to old state at certain indices.
+
+        Args:
+            old_states(list): packed decoder states
+                (L x B x H, L x B x H)
+
+            new_states: packed decoder states
+                (L x B x H, L x B x H)
+
+            ids (list): List of indices to copy states at.
+
+            value (optional float): If a value should be copied instead of a state slice, a float should be provided
+
+        Returns:
+            batch of decoder states with partial copy at ids (or a specific value).
+                (L x B x H, L x B x H)
+        """
+        for state_id in range(len(old_states)):
+            if value is None:
+                old_states[state_id][:, ids, :] = new_states[state_id][:, ids, :]
+            else:
+                old_states[state_id][:, ids, :] *= 0.0
+                old_states[state_id][:, ids, :] += value
+
+        return old_states
+
 
 class RNNTJoint(rnnt_abstract.AbstractRNNTJoint, Exportable):
     """A Recurrent Neural Network Transducer Joint Network (RNN-T Joint Network).
@@ -644,17 +697,17 @@ class RNNTJoint(rnnt_abstract.AbstractRNNTJoint, Exportable):
             }
 
     def _prepare_for_export(self, **kwargs):
-        self.freeze()
         self._fuse_loss_wer = False
         self.log_softmax = False
+        super()._prepare_for_export(**kwargs)
 
-    def input_example(self):
+    def input_example(self, max_batch=1, max_dim=8192):
         """
         Generates input examples for tracing etc.
         Returns:
             A tuple of input examples.
         """
-        B, T, U = 1, 8192, 1
+        B, T, U = max_batch, max_dim, max_batch
         encoder_outputs = torch.randn(B, self.encoder_hidden, T).to(next(self.parameters()).device)
         decoder_outputs = torch.randn(B, self.pred_hidden, U).to(next(self.parameters()).device)
         return (encoder_outputs, decoder_outputs)
@@ -664,11 +717,6 @@ class RNNTJoint(rnnt_abstract.AbstractRNNTJoint, Exportable):
         """Implement this method to return a set of input names disabled for export"""
         return set(["encoder_lengths", "transcripts", "transcript_lengths", "compute_wer"])
 
-    @property
-    def disabled_deployment_output_names(self):
-        """Implement this method to return a set of output names disabled for export"""
-        return set()
-
     def __init__(
         self,
         jointnet: Dict[str, Any],
@@ -1030,3 +1078,49 @@ class RNNTJoint(rnnt_abstract.AbstractRNNTJoint, Exportable):
 
     def set_fused_batch_size(self, fused_batch_size):
         self._fused_batch_size = fused_batch_size
+
+
+class RNNTDecoderJoint(torch.nn.Module, Exportable):
+    """
+    Utility class to export Decoder+Joint as a single module
+    """
+
+    def __init__(self, decoder, joint):
+        super().__init__()
+        self.decoder = decoder
+        self.joint = joint
+
+    @property
+    def input_types(self):
+        state_type = NeuralType(('D', 'B', 'D'), ElementType())
+        mytypes = {
+            'encoder_outputs': NeuralType(('B', 'D', 'T'), AcousticEncodedRepresentation()),
+            "targets": NeuralType(('B', 'T'), LabelsType()),
+            "target_length": NeuralType(tuple('B'), LengthsType()),
+            'input-states-1': state_type,
+            'input-states-2': state_type,
+        }
+
+        return mytypes
+
+    def input_example(self, max_batch=1, max_dim=1):
+        decoder_example = self.decoder.input_example(max_batch=max_batch, max_dim=max_dim)
+        state1, state2 = decoder_example[-1]
+        return tuple([self.joint.input_example()[0]]) + decoder_example[:2] + (state1, state2)
+
+    @property
+    def output_types(self):
+        return {
+            "outputs": NeuralType(('B', 'T', 'T', 'D'), LogprobsType()),
+            "prednet_lengths": NeuralType(tuple('B'), LengthsType()),
+            "output-states-1": NeuralType((('D', 'B', 'D')), ElementType()),
+            "output-states-2": NeuralType((('D', 'B', 'D')), ElementType()),
+        }
+
+    def forward(self, encoder_outputs, decoder_inputs, decoder_lengths, state_h, state_c):
+        decoder_outputs = self.decoder(decoder_inputs, decoder_lengths, (state_h, state_c))
+        decoder_output = decoder_outputs[0]
+        decoder_length = decoder_outputs[1]
+        state_h, state_c = decoder_outputs[2][0], decoder_outputs[2][1]
+        joint_output = self.joint(encoder_outputs, decoder_output)
+        return (joint_output, decoder_length, state_h, state_c)
