@@ -12,13 +12,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import logging
-
-from nemo.collections.asr.parts.numba.numba_utils import numba_cuda_is_supported
 from nemo.collections.asr.parts.numba.rnnt_loss.rnnt_pytorch import RNNTLossNumba
-
-# Prevent Numba CUDA logs from showing at info level
-cuda_logger = logging.getLogger('numba.cuda.cudadrv.driver')
-cuda_logger.setLevel(logging.ERROR)  # only show error
-
-__NUMBA_MINIMUM_VERSION__ = "0.53.0"
