@@ -39,13 +39,20 @@ import random
 import librosa
 import numpy as np
 import soundfile as sf
-from kaldiio.matio import read_kaldi
-from kaldiio.utils import open_like_kaldi
-from pydub import AudioSegment as Audio
-from pydub.exceptions import CouldntDecodeError
 
 from nemo.utils import logging
 
+# TODO @blisc: Perhaps refactor instead of import guarding
+HAVE_KALDI_PYDUB = True
+try:
+    from kaldiio.matio import read_kaldi
+    from kaldiio.utils import open_like_kaldi
+    from pydub import AudioSegment as Audio
+    from pydub.exceptions import CouldntDecodeError
+except ModuleNotFoundError:
+    HAVE_KALDI_PYDUB = False
+
+
 available_formats = sf.available_formats()
 sf_supported_formats = ["." + i.lower() for i in available_formats.keys()]
 
@@ -149,7 +156,7 @@ class AudioSegment(object):
                     f"Loading {audio_file} via SoundFile raised RuntimeError: `{e}`. "
                     f"NeMo will fallback to loading via pydub."
                 )
-        elif isinstance(audio_file, str) and audio_file.strip()[-1] == "|":
+        elif HAVE_KALDI_PYDUB and isinstance(audio_file, str) and audio_file.strip()[-1] == "|":
             f = open_like_kaldi(audio_file, "rb")
             sample_rate, samples = read_kaldi(f)
             if offset > 0:
@@ -160,7 +167,7 @@ class AudioSegment(object):
                 abs_max_value = np.abs(samples).max()
                 samples = np.array(samples, dtype=np.float) / abs_max_value
 
-        if samples is None:
+        if HAVE_KALDI_PYDUB and samples is None:
             try:
                 samples = Audio.from_file(audio_file)
                 sample_rate = samples.frame_rate
@@ -172,8 +179,12 @@ class AudioSegment(object):
                     seconds = duration * 1000
                     samples = samples[: int(seconds)]
                 samples = np.array(samples.get_array_of_samples())
-            except CouldntDecodeError as e:
-                logging.error(f"Loading {audio_file} via pydub raised CouldntDecodeError: `{e}`.")
+            except CouldntDecodeError as err:
+                logging.error(f"Loading {audio_file} via pydub raised CouldntDecodeError: `{err}`.")
+
+        if samples is None:
+            libs = "soundfile, kaldiio, and pydub" if HAVE_KALDI_PYDUB else "soundfile"
+            raise Exception(f"Your audio file {audio_file} could not be decoded. We tried using {libs}.")
 
         return cls(samples, sample_rate, target_sr=target_sr, trim=trim, orig_sr=orig_sr)
 
