@@ -173,6 +173,11 @@ class AutoTokenizer(TokenizerSpec):
             setattr(self, k, getattr(self.tokenizer, k, None))
         return num_tokens_added
 
+    @property
+    def additional_special_tokens_ids(self):
+        """Returns a list of the additional special tokens (excluding bos, eos, pad, unk). Used to return sentinel tokens for e.g. T5."""
+        return [self.token_to_id(token) for token in self.additional_special_tokens]
+
     def text_to_tokens(self, text):
         tokens = self.tokenizer.tokenize(text)
         return tokens
@@ -203,6 +208,11 @@ class AutoTokenizer(TokenizerSpec):
         text = self.tokens_to_text(tokens_clean)
         return text
 
+    @property
+    def vocab(self):
+        id2vocab = {v: k for k, v in self.tokenizer.vocab.items()}
+        return [id2vocab[i] for i in range(len(id2vocab))]
+
     @property
     def pad_id(self):
         return self.tokens_to_ids([getattr(self, 'pad_token')])[0]
