@@ -15,12 +15,13 @@
 """BERT model."""
 
 import torch
-from apex.transformer import tensor_parallel
+from apex.transformer import parallel_state, tensor_parallel
+from apex.transformer.enums import AttnMaskType
+from apex.transformer.tensor_parallel.layers import set_tensor_model_parallel_attributes
 
-from nemo.collections.nlp.modules.common.megatron.enums import AttnMaskType
-from nemo.collections.nlp.modules.common.megatron.fused_layer_norm import MixedFusedLayerNorm as LayerNorm
 from nemo.collections.nlp.modules.common.megatron.language_model import get_language_model, parallel_lm_logits
 from nemo.collections.nlp.modules.common.megatron.module import MegatronModule
+from nemo.collections.nlp.modules.common.megatron.transformer import get_layer_norm
 from nemo.collections.nlp.modules.common.megatron.utils import (
     erf_gelu,
     get_linear_layer,
@@ -67,22 +68,22 @@ class BertLMHead(MegatronModule):
         parallel_output: whether output logits being distributed or not.
     """
 
-    def __init__(self, mpu_vocab_size, hidden_size, init_method, layernorm_epsilon, parallel_output):
+    def __init__(
+        self, mpu_vocab_size, hidden_size, init_method, layernorm_epsilon, parallel_output, use_openai_gelu, onnx_safe
+    ):
 
         super(BertLMHead, self).__init__()
 
-        args = get_args()
-
         self.bias = torch.nn.Parameter(torch.zeros(mpu_vocab_size))
-        parallel_state.set_tensor_model_parallel_attributes(self.bias, True, 0, 1)
+        set_tensor_model_parallel_attributes(self.bias, True, 0, 1)
         self.parallel_output = parallel_output
 
         self.dense = get_linear_layer(hidden_size, hidden_size, init_method)
-        self.layernorm = LayerNorm(hidden_size, eps=layernorm_epsilon)
+        self.layernorm = get_layer_norm(hidden_size, eps=layernorm_epsilon)
         self.gelu = torch.nn.functional.gelu
-        if args.openai_gelu:
+        if use_openai_gelu:
             self.gelu = openai_gelu
-        elif args.onnx_safe:
+        elif onnx_safe:
             self.gelu = erf_gelu
 
     def forward(self, hidden_states, word_embeddings_weight):
@@ -118,43 +119,91 @@ class BertModel(MegatronModule):
     """Bert Language model."""
 
     def __init__(
-        self, num_tokentypes=2, add_binary_head=True, parallel_output=True, pre_process=True, post_process=True
+        self,
+        vocab_size,
+        hidden_size,
+        max_position_embeddings,
+        num_layers,
+        num_attention_heads,
+        ffn_hidden_size,
+        apply_query_key_layer_scaling=True,
+        kv_channels=None,
+        num_tokentypes=0,
+        parallel_output=True,
+        pre_process=True,
+        post_process=True,
+        init_method_std=0.02,
+        fp16_lm_cross_entropy=False,
+        use_cpu_initialization=False,
+        hidden_dropout=0.1,
+        precision=16,
+        fp32_residual_connection=False,
+        activations_checkpoint_method=None,
+        activations_checkpoint_num_layers=1,
+        layernorm_epsilon=1e-5,
+        bias_gelu_fusion=True,
+        openai_gelu=False,
+        onnx_safe=False,
+        add_binary_head=True,
     ):
         super(BertModel, self).__init__()
-        args = get_args()
-
-        self.fp16_lm_cross_entropy = args.fp16_lm_cross_entropy
+        # args = get_args()
+        self.fp16_lm_cross_entropy = fp16_lm_cross_entropy
         self.add_binary_head = add_binary_head
         self.parallel_output = parallel_output
         self.pre_process = pre_process
         self.post_process = post_process
 
-        init_method = init_method_normal(args.init_method_std)
-        scaled_init_method = scaled_init_method_normal(args.init_method_std, args.num_layers)
+        init_method = init_method_normal(init_method_std)
+        scaled_init_method = scaled_init_method_normal(init_method_std, num_layers)
 
         self.language_model, self._language_model_key = get_language_model(
+            vocab_size=vocab_size,
+            hidden_size=hidden_size,
+            hidden_dropout=hidden_dropout,
             num_tokentypes=num_tokentypes,
+            max_position_embeddings=max_position_embeddings,
+            num_layers=num_layers,
+            num_attention_heads=num_attention_heads,
+            apply_query_key_layer_scaling=apply_query_key_layer_scaling,
+            kv_channels=kv_channels,
+            ffn_hidden_size=ffn_hidden_size,
             add_pooler=self.add_binary_head,
             encoder_attn_mask_type=AttnMaskType.padding,
             init_method=init_method,
             scaled_init_method=scaled_init_method,
             pre_process=self.pre_process,
             post_process=self.post_process,
+            init_method_std=init_method_std,
+            use_cpu_initialization=use_cpu_initialization,
+            precision=precision,
+            fp32_residual_connection=fp32_residual_connection,
+            activations_checkpoint_method=activations_checkpoint_method,
+            activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+            layernorm_epsilon=layernorm_epsilon,
+            bias_gelu_fusion=bias_gelu_fusion,
+            openai_gelu=openai_gelu,
+            onnx_safe=onnx_safe,
+        )
+
+        self.initialize_word_embeddings(
+            init_method=init_method_normal(init_method_std), vocab_size=vocab_size, hidden_size=hidden_size
         )
 
-        self.initialize_word_embeddings(init_method_normal)
         if self.post_process:
             self.lm_head = BertLMHead(
                 self.word_embeddings_weight().size(0),
-                args.hidden_size,
+                hidden_size,
                 init_method,
-                args.layernorm_epsilon,
+                layernorm_epsilon,
                 parallel_output,
+                openai_gelu,
+                onnx_safe,
             )
             self._lm_head_key = 'lm_head'
             self.binary_head = None
             if self.add_binary_head:
-                self.binary_head = get_linear_layer(args.hidden_size, 2, init_method)
+                self.binary_head = get_linear_layer(hidden_size, 2, init_method)
                 self._binary_head_key = 'binary_head'
 
     def set_input_tensor(self, input_tensor):
