@@ -60,12 +60,15 @@ class NLPModel(ModelPT, Exportable):
         self._save_restore_connector = NLPSaveRestoreConnector()
         self.set_world_size(trainer)
         if not HAVE_APEX:
-            logging.warning("Apex was not found. Using model parallel or megatron models will error out.")
+            logging.warning(
+                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+            )
 
     def register_artifact(
         self, config_path: str, src: str, verify_src_exists: bool = False,
     ):
-        """ Overrides ModelPT register_artifact default behavior. NLP models usually need artifacts that are optional."""
+        """ Overrides ModelPT register_artifact default behavior.
+        NLP models usually need artifacts that are optional."""
         return super().register_artifact(config_path, src, verify_src_exists=verify_src_exists)
 
     @rank_zero_only
@@ -243,13 +246,24 @@ class NLPModel(ModelPT, Exportable):
             if cls.CHECKPOINT_HYPER_PARAMS_KEY not in checkpoint:
                 checkpoint[cls.CHECKPOINT_HYPER_PARAMS_KEY] = {}
             # override the hparams with values that were passed in
+            cfg = checkpoint[cls.CHECKPOINT_HYPER_PARAMS_KEY].get('cfg', checkpoint[cls.CHECKPOINT_HYPER_PARAMS_KEY])
             # TODO: can we do this without overriding?
             config_kwargs = kwargs.copy()
             if 'trainer' in config_kwargs:
                 config_kwargs.pop('trainer')
-            checkpoint[cls.CHECKPOINT_HYPER_PARAMS_KEY].update(config_kwargs)
+            cfg.update(config_kwargs)
 
-            model = cls._load_model_state(checkpoint, strict=strict, **kwargs)
+            if cfg.get('megatron_amp_O2', False):
+                new_state_dict = {}
+                for key in checkpoint['state_dict'].keys():
+                    new_key = key.replace('model.', 'model.module.', 1)
+                    new_state_dict[new_key] = checkpoint['state_dict'][key]
+                checkpoint['state_dict'] = new_state_dict
+
+            if 'cfg' in kwargs:
+                model = cls._load_model_state(checkpoint, strict=strict, **kwargs)
+            else:
+                model = cls._load_model_state(checkpoint, strict=strict, cfg=cfg, **kwargs)
             checkpoint = model
 
         finally:
