@@ -18,13 +18,11 @@ from nemo.collections.nlp.modules.common import (
     BertEncoder,
     BertModule,
     DistilBertEncoder,
-    MegatronBertEncoder,
     RobertaEncoder,
     SequenceClassifier,
     SequenceRegression,
     SequenceTokenClassifier,
     get_lm_model,
-    get_megatron_lm_models_list,
     get_pretrained_lm_models_list,
     get_tokenizer,
     get_tokenizer_list,
