@@ -71,14 +71,15 @@ class BertModule(NeuralModule, Exportable):
         self.load_state_dict(model_dict)
         logging.info(f"Weights for {type(self).__name__} restored from {restore_path}")
 
-    def input_example(self):
+    def input_example(self, max_batch=1, max_dim=256):
         """
         Generates input examples for tracing etc.
         Returns:
             A tuple of input examples.
         """
         sample = next(self.parameters())
-        input_ids = torch.randint(low=0, high=2048, size=(2, 16), device=sample.device)
-        token_type_ids = torch.randint(low=0, high=1, size=(2, 16), device=sample.device)
-        attention_mask = torch.randint(low=0, high=1, size=(2, 16), device=sample.device)
+        sz = (max_batch, max_dim)
+        input_ids = torch.randint(low=0, high=2048, size=sz, device=sample.device)
+        token_type_ids = torch.randint(low=0, high=1, size=sz, device=sample.device)
+        attention_mask = torch.randint(low=0, high=1, size=sz, device=sample.device)
         return tuple([input_ids, token_type_ids, attention_mask])
