@@ -13,9 +13,12 @@
 # limitations under the License.
 
 """Transformer based language model."""
+import math
 
 import torch
+import torch.nn as nn
 import torch.nn.functional as F
+import torch.nn.init as init
 from apex.transformer import parallel_state, tensor_parallel
 from apex.transformer.enums import AttnMaskType, LayerType
 
@@ -71,8 +74,12 @@ def get_language_model(
     activations_checkpoint_num_layers=1,
     layernorm_epsilon=1e-5,
     bias_gelu_fusion=True,
+    persist_layer_norm=False,
     openai_gelu=False,
     onnx_safe=False,
+    use_soft_prompts=False,
+    num_prompt_tokens=10,
+    prompt_tags=None,
 ):
     """Build language model and return along with the key to save."""
 
@@ -115,8 +122,12 @@ def get_language_model(
         activations_checkpoint_num_layers=activations_checkpoint_num_layers,
         layernorm_epsilon=layernorm_epsilon,
         bias_gelu_fusion=bias_gelu_fusion,
+        persist_layer_norm=persist_layer_norm,
         openai_gelu=openai_gelu,
         onnx_safe=onnx_safe,
+        use_soft_prompts=use_soft_prompts,
+        num_prompt_tokens=num_prompt_tokens,
+        prompt_tags=prompt_tags,
     )
     # key used for checkpoints.
     language_model_key = 'language_model'
@@ -141,7 +152,7 @@ class Pooler(MegatronModule):
         self.dense = get_linear_layer(hidden_size, hidden_size, init_method)
 
     def forward(self, hidden_states, sequence_index=0):
-        # hidden_states: [b, s, h]
+        # hidden_states: [b, s, h]prompt_embeddings
         # sequence_index: index of the token to pool.
         pooled = hidden_states[:, sequence_index, :]
         pooled = self.dense(pooled)
@@ -295,6 +306,200 @@ class Embedding(MegatronModule):
                 )
 
 
+class PromptEmbedding(MegatronModule):
+    """Prompt embeddings
+
+    Arugments:
+        init_from_prompt_text: Whether to intialize prompt embeddings
+                               from from certain lm embeddings
+                               corresponding to a prompt string
+        hidden_size: hidden size should match lm embedding size
+        num_prompt_tokens: length of prompt initalized from torch init method
+        position_embedding_weights: embedding vectors for positions 0 through 
+                                    num_prompt_tokens
+        word_embedding_weights: token embedding vectors for text init option
+        init_method: pytorch init method
+        embedding_weights: token embeddings from prompt text
+        prompt_embedding_dropout_prob: dropout probablity
+    """
+
+    def __init__(
+        self,
+        init_from_prompt_text,
+        hidden_size,
+        num_prompt_tokens,
+        position_embedding_weights=None,
+        word_embedding_weights=None,
+        init_method=init.xavier_normal_,
+        prompt_embedding_dropout_prob=0.1,
+    ):
+        super().__init__()
+
+        self.hidden_size = hidden_size
+        self.num_prompt_tokens = num_prompt_tokens
+        self.prompt_embeddings = torch.nn.Embedding(self.num_prompt_tokens, self.hidden_size)
+        init_method(self.prompt_embeddings.weight)
+
+        if init_from_prompt_text:
+
+            # Set embedding weights to be embeddings from prompt tokens
+            self.prompt_embeddings.weight = nn.Parameter(word_embedding_weights)
+
+        self._prompt_embeddings_key = 'prompt_embeddings'
+
+        self.position_embeddings = torch.nn.Embedding(self.num_prompt_tokens, self.hidden_size)
+        self._position_embeddings_key = 'position_embeddings'
+
+        if position_embedding_weights != None:
+            self.position_embeddings.weight = nn.Parameter(position_embedding_weights)
+
+        self.prompt_ids = torch.tensor([i for i in range(self.num_prompt_tokens)])
+        self.embedding_dropout = torch.nn.Dropout(prompt_embedding_dropout_prob)
+
+    def forward(self, tokentype_ids=None):
+        # Embeddings.
+        device = next(self.prompt_embeddings.parameters()).device
+        prompt_embeddings = self.prompt_embeddings(self.prompt_ids.to(device))
+        position_embeddings = self.position_embeddings(self.prompt_ids.to(device))
+        embeddings = prompt_embeddings + position_embeddings
+
+        # Dropout.
+        embeddings = self.embedding_dropout(embeddings)
+
+        return embeddings
+
+    # These save and load methods don't actually seem to be called during training or when restoring the model
+    # But I've added them because the other transformer submodules have them
+    def state_dict_for_save_checkpoint(self, destination=None, prefix='', keep_vars=False):
+        """For easy load."""
+        state_dict_ = {}
+        state_dict_[self._prompt_embeddings_key] = self.prompt_embeddings.state_dict(destination, prefix, keep_vars)
+        state_dict_[self._position_embeddings_key] = self.position_embeddings.state_dict(
+            destination, prefix, keep_vars
+        )
+
+        return state_dict_
+
+    def load_state_dict(self, state_dict, strict=True):
+        """Customized load."""
+
+        # Word embedding.
+        if self._prompt_embeddings_key in state_dict:
+            state_dict_ = state_dict[self._word_embeddings_key]
+        else:
+            # for backward compatibility.
+            state_dict_ = {}
+            for key in state_dict.keys():
+                if 'prompt_embeddings' in key:
+                    state_dict_[key.split('prompt_embeddings.')[1]] = state_dict[key]
+        self.prompt_embeddings.load_state_dict(state_dict_, strict=strict)
+
+        # Position embedding.
+        if self._position_embeddings_key in state_dict:
+            state_dict_ = state_dict[self._position_embeddings_key]
+        else:
+            # for backward compatibility.
+            state_dict_ = {}
+            for key in state_dict.keys():
+                if 'position_embeddings' in key:
+                    state_dict_[key.split('position_embeddings.')[1]] = state_dict[key]
+        self.position_embeddings.load_state_dict(state_dict_, strict=strict)
+
+
+class PromptTable(torch.nn.Module):
+    def __init__(
+        self, prompt_tags, num_prompt_tokens, hidden_size,
+    ):
+        super().__init__()
+
+        self.num_prompt_tokens = num_prompt_tokens
+        self.hidden_size = hidden_size
+        self.prompt_table = torch.nn.ModuleDict()
+
+        if prompt_tags:
+            for tag in enumerate(prompt_tags):
+                _, tag = tag
+                self.prompt_table[tag] = PromptEmbedding(
+                    init_from_prompt_text=False,
+                    hidden_size=self.hidden_size,
+                    num_prompt_tokens=self.num_prompt_tokens,
+                )
+
+    def forward(self, prompt_tag):
+        return self.prompt_table[prompt_tag]()
+
+    def remove_prompt(self, prompt_tag):
+        del self.prompt_table[prompt_tag]
+
+    def init_prompt_from_random(self, prompt_tag, position_embeddings):
+        """Add new soft prompt to be tuned.
+           Intialize prompt weights using pytorch init method
+
+        """
+        device = next(position_embeddings.parameters()).device
+        position_weights = (
+            position_embeddings(torch.tensor([i for i in range(self.num_prompt_tokens)]).to(device)).detach().clone()
+        )
+
+        # Initalize prompt embeddings from a pytorch random init method
+        prompt_embeddings = PromptEmbedding(
+            init_from_prompt_text=False,
+            hidden_size=self.hidden_size,
+            num_prompt_tokens=self.num_prompt_tokens,
+            position_embedding_weights=position_weights,
+        )
+
+        self.prompt_table[prompt_tag] = prompt_embeddings
+
+    def init_prompt_from_text(self, prompt_tag, init_token_ids, word_embeddings, position_embeddings):
+        """Add new soft prompt to be tuned.
+           Intialize prompt weights from existing embeddings from specific vocab tokens.
+
+        """
+        # Trim or iterate until num_text_tokens matches num_prompt_tokens
+        num_text_tokens = len(init_token_ids)
+        num_prompt_tokens = self.num_prompt_tokens
+
+        if num_text_tokens > num_prompt_tokens:
+            init_token_ids = init_token_ids[:num_prompt_tokens]
+        elif num_text_tokens < num_prompt_tokens:
+            num_reps = math.ceil(num_prompt_tokens / num_text_tokens)
+            init_token_ids = init_token_ids * num_reps
+        init_token_ids = init_token_ids[:num_prompt_tokens]
+
+        # Use a copy of token embedding weights to initalize the prompt embeddings
+        device = next(word_embeddings.parameters()).device
+        embedding_weights = word_embeddings(torch.tensor(init_token_ids, device=device)).detach().clone()
+        position_weights = (
+            position_embeddings(torch.tensor([i for i in range(self.num_prompt_tokens)], device=device))
+            .detach()
+            .clone()
+        )
+
+        prompt_embeddings = PromptEmbedding(
+            init_from_prompt_text=True,
+            hidden_size=self.hidden_size,
+            num_prompt_tokens=self.num_prompt_tokens,
+            word_embedding_weights=embedding_weights,
+            position_embedding_weights=position_weights,
+        )
+
+        self.prompt_table[prompt_tag] = prompt_embeddings
+
+    def load_state_dict(self, state_dict_, strict):
+        for prompt_tag in self.prompt_table:
+            self.prompt_table[prompt_tag].load_state_dict(state_dict_[prompt_tag], strict=strict)
+
+    def state_dict_for_save_checkpoint(self, destination=None, prefix='', keep_vars=False):
+        prompt_state_dict_ = {}
+        for prompt_tag in self.prompt_table:
+            prompt_state_dict_[prompt_tag] = self.prompt_table[prompt_tag].state_dict_for_save_checkpoint(
+                destination, prefix, keep_vars
+            )
+
+        return prompt_state_dict_
+
+
 class TransformerLanguageModel(MegatronModule):
     """Transformer language model.
 
@@ -335,8 +540,12 @@ class TransformerLanguageModel(MegatronModule):
         activations_checkpoint_num_layers=1,
         layernorm_epsilon=1e-5,
         bias_gelu_fusion=True,
+        persist_layer_norm=False,
         openai_gelu=False,
         onnx_safe=False,
+        use_soft_prompts=False,
+        num_prompt_tokens=10,
+        prompt_tags=None,
     ):
         super(TransformerLanguageModel, self).__init__()
 
@@ -354,6 +563,9 @@ class TransformerLanguageModel(MegatronModule):
         self.add_pooler = add_pooler
         self.hidden_dropout = hidden_dropout
         self.output_layer_init_method = output_layer_init_method
+        self.use_soft_prompts = use_soft_prompts
+        self.prompt_tags = prompt_tags
+        self.num_prompt_tokens = num_prompt_tokens
 
         if kv_channels is None:
 
@@ -375,6 +587,13 @@ class TransformerLanguageModel(MegatronModule):
             )
             self._embedding_key = 'embedding'
 
+        # Soft Prompts
+        if self.use_soft_prompts:
+            self.prompt_table = PromptTable(
+                prompt_tags=self.prompt_tags, num_prompt_tokens=self.num_prompt_tokens, hidden_size=self.hidden_size,
+            )
+            self._prompt_table_key = 'prompt_table'
+
         # Transformer.
         self.encoder = ParallelTransformer(
             init_method=self.init_method,
@@ -396,6 +615,7 @@ class TransformerLanguageModel(MegatronModule):
             hidden_dropout=hidden_dropout,
             use_cpu_initialization=use_cpu_initialization,
             bias_gelu_fusion=bias_gelu_fusion,
+            persist_layer_norm=persist_layer_norm,
             openai_gelu=openai_gelu,
             onnx_safe=onnx_safe,
         )
@@ -427,6 +647,7 @@ class TransformerLanguageModel(MegatronModule):
                 hidden_dropout=hidden_dropout,
                 use_cpu_initialization=use_cpu_initialization,
                 bias_gelu_fusion=bias_gelu_fusion,
+                persist_layer_norm=persist_layer_norm,
                 openai_gelu=openai_gelu,
                 onnx_safe=onnx_safe,
             )
@@ -447,6 +668,7 @@ class TransformerLanguageModel(MegatronModule):
         enc_input_ids,
         enc_position_ids,
         enc_attn_mask,
+        prompt_tags=None,
         dec_input_ids=None,
         dec_position_ids=None,
         dec_attn_mask=None,
@@ -456,12 +678,21 @@ class TransformerLanguageModel(MegatronModule):
         get_key_value=False,
         pooling_sequence_index=0,
         enc_hidden_states=None,
-        output_enc_hidden=False,
+        output_enc_hidden_only=False,
     ):
         # Embeddings.
         if self.pre_process:
             embedding_output = self.embedding(enc_input_ids, enc_position_ids, tokentype_ids=tokentype_ids)
-            encoder_input = embedding_output
+
+            # Soft prompts
+            if self.use_soft_prompts and prompt_tags:
+                prompt_embeddings = [self.prompt_table(tag) for tag in prompt_tags]
+                prompt_embeddings = torch.stack(prompt_embeddings)
+
+                encoder_input = torch.cat((prompt_embeddings, embedding_output), dim=1)
+
+            else:
+                encoder_input = embedding_output
         else:
             encoder_input = None
 
@@ -477,10 +708,10 @@ class TransformerLanguageModel(MegatronModule):
             if self.add_pooler:
                 pooled_output = self.pooler(encoder_output, pooling_sequence_index)
 
-        # output_enc_hidden refers to when we just need the encoder's
+        # output_enc_hidden_only refers to when we just need the encoder's
         # output. For example, it is helpful to compute
         # similarity between two sequences by average pooling
-        if not self.add_decoder or output_enc_hidden:
+        if not self.add_decoder or output_enc_hidden_only:
             if self.add_pooler and self.post_process:
                 return encoder_output, pooled_output
             else:
@@ -511,6 +742,12 @@ class TransformerLanguageModel(MegatronModule):
             state_dict_[self._embedding_key] = self.embedding.state_dict_for_save_checkpoint(
                 destination, prefix, keep_vars
             )
+
+        if self.use_soft_prompts:
+            state_dict_[self._prompt_table_key] = self.prompt_table.state_dict_for_save_checkpoint(
+                destination, prefix, keep_vars
+            )
+
         state_dict_[self._encoder_key] = self.encoder.state_dict_for_save_checkpoint(destination, prefix, keep_vars)
         if self.post_process:
             if self.add_pooler:
@@ -539,9 +776,17 @@ class TransformerLanguageModel(MegatronModule):
                         state_dict_[key] = state_dict[key]
             self.embedding.load_state_dict(state_dict_, strict=strict)
 
+        # Prompt Table
+        if self.use_soft_prompts:
+            if self._prompt_table_key in state_dict:
+                state_dict_ = state_dict[self._prompt_table_key]
+
+                self.prompt_table.load_state_dict(state_dict_, strict=strict)
+
         # Encoder.
         if self._encoder_key in state_dict:
             state_dict_ = state_dict[self._encoder_key]
+
         # for backward compatibility.
         elif 'transformer' in state_dict:
             state_dict_ = state_dict['transformer']
@@ -572,3 +817,30 @@ class TransformerLanguageModel(MegatronModule):
         if self.add_decoder:
             assert 'decoder' in state_dict, 'could not find data for pooler in the checkpoint'
             self.decoder.load_state_dict(state_dict[self._decoder_key], strict=strict)
+
+    def _init_prompt_from_random(self, prompt_tag):
+        """Add new soft prompt to be tuned.
+           Intialize prompt weights using pytorch init method
+
+        """
+
+        if not hasattr(self, 'prompt_table'):
+            raise AttributeError('Please set "use_soft_prompts" in the config to True')
+
+        self.prompt_table.init_prompt_from_random(prompt_tag, self.embedding.position_embeddings)
+
+    def _init_prompt_from_text(self, prompt_tag, init_token_ids):
+        """Add new soft prompt to be tuned.
+           Intialize prompt weights from existing embeddings from specific vocab tokens.
+
+        """
+
+        if not hasattr(self, 'prompt_table'):
+            raise AttributeError('Please set "use_soft_prompts" in the config to True')
+
+        self.prompt_table.init_prompt_from_text(
+            prompt_tag,
+            init_token_ids,
+            word_embeddings=self.embedding.word_embeddings,
+            position_embeddings=self.embedding.position_embeddings,
+        )
