@@ -16,17 +16,23 @@ import os
 import shutil
 import tempfile
 from collections import defaultdict
-from typing import Any, Dict, List, Optional, Union
+from contextlib import contextmanager
+from typing import Any, Callable, Dict, Generator, List, Mapping, Optional, Union
 
+import pytorch_lightning as pl
 import torch
 from pytorch_lightning.overrides import LightningDistributedModule
 from pytorch_lightning.plugins.environments.cluster_environment import ClusterEnvironment
 from pytorch_lightning.plugins.io.checkpoint_plugin import CheckpointIO
+from pytorch_lightning.plugins.precision import NativeMixedPrecisionPlugin
 from pytorch_lightning.plugins.training_type.ddp import DDPPlugin
 from pytorch_lightning.utilities.types import _PATH
+from torch.distributed.algorithms.ddp_comm_hooks.debugging_hooks import noop_hook
 from torch.nn.parallel import DistributedDataParallel
 
+from nemo.collections.nlp.modules.common.megatron.module import Float16Module
 from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
+from nemo.core.optim import MasterOptimizerWrapper
 from nemo.utils import AppState, logging
 
 try:
@@ -41,6 +47,10 @@ except (ImportError, ModuleNotFoundError):
 
 class NLPDDPPlugin(DDPPlugin):
     """ DDP plugin for Pytorch Lightning. Needed to customize DDP for model parallel models.
+
+    Args:
+        no_ddp_communication_hook: Disable DDP communication hook when using AMP-O2
+        with FP32 gradient accumulation.
     """
 
     accelerator = "ddp"
@@ -52,12 +62,16 @@ class NLPDDPPlugin(DDPPlugin):
         cluster_environment: ClusterEnvironment = None,
         sync_batchnorm: bool = False,
         checkpoint_io: Optional[CheckpointIO] = None,
+        no_ddp_communication_hook: bool = False,
         **kwargs: Union[Any, Dict[str, Any]],
     ) -> None:
         super().__init__(parallel_devices, num_nodes, cluster_environment, checkpoint_io, sync_batchnorm, **kwargs)
 
         if not HAVE_APEX:
-            logging.warning("Apex was not found. Using model parallel or megatron models will error out.")
+            logging.warning(
+                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+            )
+        self.no_ddp_communication_hook = no_ddp_communication_hook
 
     def setup_distributed(self, global_rank: int = None, world_size: int = None) -> None:
         # call PTL init ddp
@@ -92,6 +106,13 @@ class NLPDDPPlugin(DDPPlugin):
                 **self._ddp_kwargs,
             )
 
+            if self.no_ddp_communication_hook:
+                # When using custom gradient accumulation and allreduce, disable
+                # DDP communication hook that works on the gradient bucket.
+                # Instead, use the custom gradient function and communication hook,
+                # which is defined in the master optimizer wrapper.
+                self._model.register_comm_hook(None, noop_hook)
+
         else:
             super().configure_ddp()
 
@@ -123,6 +144,19 @@ class NLPDDPPlugin(DDPPlugin):
         filepath = self._inject_model_parallel_rank(filepath)
         return super().save_checkpoint(checkpoint, filepath)
 
+    def load_model_state_dict(self, checkpoint: Mapping[str, Any]) -> None:
+        # Release strict state dict matching when using Megatron AMP-O2 to skip matching
+        # half-precision module wrapper module.
+        if hasattr(self.lightning_module, 'model'):
+            if isinstance(self.lightning_module.model, Float16Module):
+                new_state_dict = {}
+                for key in checkpoint['state_dict'].keys():
+                    new_key = key.replace('model.', 'model.module.', 1)
+                    new_state_dict[new_key] = checkpoint['state_dict'][key]
+                checkpoint['state_dict'] = new_state_dict
+
+        self.lightning_module.load_state_dict(checkpoint["state_dict"])
+
     def remove_checkpoint(self, filepath: _PATH) -> None:
         # PTL override to accomodate model parallel checkpoints
         filepath = self._inject_model_parallel_rank(filepath)
@@ -171,7 +205,9 @@ class NLPSaveRestoreConnector(SaveRestoreConnector):
     def __init__(self) -> None:
         super().__init__()
         if not HAVE_APEX:
-            logging.warning("Apex was not found. Using model parallel or megatron models will error out.")
+            logging.warning(
+                "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
+            )
 
     def save_to(self, model, save_path: str):
         app_state = AppState()
@@ -223,7 +259,13 @@ class GradScaler(torch.cuda.amp.GradScaler):
     """
 
     def __init__(
-        self, init_scale=2.0 ** 16, growth_factor=2.0, backoff_factor=0.5, growth_interval=2000, enabled=True
+        self,
+        init_scale=2.0 ** 16,
+        growth_factor=2.0,
+        backoff_factor=0.5,
+        growth_interval=2000,
+        enabled=True,
+        hysteresis=1,
     ):
         super().__init__(
             init_scale=init_scale,
@@ -232,6 +274,9 @@ class GradScaler(torch.cuda.amp.GradScaler):
             growth_interval=growth_interval,
             enabled=enabled,
         )
+        self.optimizer_update_skipped: Optional[bool] = None
+        self.hysteresis = hysteresis
+        self._hysteresis_tracker = self.hysteresis
 
     def _maybe_opt_step(self, optimizer, optimizer_state, *args, **kwargs):
         retval = None
@@ -244,27 +289,17 @@ class GradScaler(torch.cuda.amp.GradScaler):
 
         if found_inf.item() == 0:
             retval = optimizer.step(*args, **kwargs)
+            self.optimizer_update_skipped = False
+        else:
+            self.optimizer_update_skipped = True
         return retval
 
     def update(self, new_scale=None):
         """
-        Updates the scale factor.
-
-        If any optimizer steps were skipped the scale is multiplied by ``backoff_factor``
-        to reduce it. If ``growth_interval`` unskipped iterations occurred consecutively,
-        the scale is multiplied by ``growth_factor`` to increase it.
-
-        Passing ``new_scale`` sets the new scale value manually. (``new_scale`` is not
-        used directly, it's used to fill GradScaler's internal scale tensor. So if
-        ``new_scale`` was a tensor, later in-place changes to that tensor will not further
-        affect the scale GradScaler uses internally.)
-
-        Args:
-            new_scale (float or :class:`torch.cuda.FloatTensor`, optional, default=None):  New scale factor.
-
-        .. warning::
-            :meth:`update` should only be called at the end of the iteration, after ``scaler.step(optimizer)`` has
-            been invoked for all optimizers used this iteration.
+        Updates to native grad scaler update function.
+        1. Check inf across model-parallel ranks.
+        2. Update hysteresis tracker.
+        3. Apply hysteresis to grad scale update.
         """
         if not self._enabled:
             return
@@ -308,14 +343,154 @@ class GradScaler(torch.cuda.amp.GradScaler):
                     )
                     found_inf_combined += found_inf
 
-            torch._amp_update_scale_(
-                _scale,
-                _growth_tracker,
-                found_inf_combined,
-                self._growth_factor,
-                self._backoff_factor,
-                self._growth_interval,
-            )
+            if found_inf_combined > 0:
+                self._hysteresis_tracker -= 1
+                if self._hysteresis_tracker <= 0:
+                    # When hysteresis becomes zero, follow the native grad scale update rule.
+                    # Increase scale and reset growth tracker
+                    torch._amp_update_scale_(
+                        _scale,
+                        _growth_tracker,
+                        found_inf_combined,
+                        self._growth_factor,
+                        self._backoff_factor,
+                        self._growth_interval,
+                    )
+                else:
+                    # Only reset the growth tracker when hysteresis is larger than zero
+                    _growth_tracker.fill_(0.0)
+            else:
+                # When no inf found, follow the native grad scale update rule.
+                # Increment growth_tracker, update scale when growth tracker reaches the interval, and
+                # reset the hysteresis tracker.
+                torch._amp_update_scale_(
+                    _scale,
+                    _growth_tracker,
+                    found_inf_combined,
+                    self._growth_factor,
+                    self._backoff_factor,
+                    self._growth_interval,
+                )
+                self._hysteresis_tracker = self.hysteresis
 
         # To prepare for next iteration, clear the data collected from optimizers this iteration.
         self._per_optimizer_states = defaultdict(torch.cuda.amp.grad_scaler._refresh_per_optimizer_state)
+
+    def state_dict(self):
+        """
+        Add hysteresis_tracker to the native functions' state_dict
+        """
+        return (
+            {
+                "scale": self.get_scale(),
+                "growth_factor": self._growth_factor,
+                "backoff_factor": self._backoff_factor,
+                "growth_interval": self._growth_interval,
+                "_growth_tracker": self._get_growth_tracker(),
+                "_hysteresis_tracker": self._hysteresis_tracker,
+            }
+            if self._enabled
+            else {}
+        )
+
+    def load_state_dict(self, state_dict):
+        """
+        Load hysteresis_tracker in addition to the state dict of the native function
+        """
+        if not self._enabled:
+            return
+
+        if len(state_dict) == 0:
+            raise RuntimeError(
+                "The source state dict is empty, possibly because it was saved "
+                "from a disabled instance of GradScaler."
+            )
+
+        self._init_scale = state_dict["scale"]
+        if self._scale is not None:
+            self._scale.fill_(state_dict["scale"])
+        self._growth_factor = state_dict["growth_factor"]
+        self._backoff_factor = state_dict["backoff_factor"]
+        self._growth_interval = state_dict["growth_interval"]
+        self._init_growth_tracker = state_dict["_growth_tracker"]
+        if self._growth_tracker is not None:
+            self._growth_tracker.fill_(state_dict["_growth_tracker"])
+        if "_hysterisis_tracker" in state_dict:
+            self._hysteresis_tracker = state_dict["_hysterisis_tracker"]
+        else:
+            self._hysteresis_tracker = 1
+
+
+class MegatronHalfPrecisionPlugin(NativeMixedPrecisionPlugin):
+    """
+    Plugin for Half (FP16 and BF16) precision training.
+    This plugin assumes the use of the optimizer with master parameters (fp32).
+    This plugin uses half-precision at all operators in the model so need of input precision
+    at each layer operator.
+
+    Args:
+        precision: Whether to use ``torch.float16`` (``16``) or ``torch.bfloat16`` (``'bf16'``).
+        device: The device for ``torch.autocast``.
+        scaler: An optional :class:`torch.cuda.amp.GradScaler` to use.
+    """
+
+    def __init__(
+        self, precision: Union[str, int], device: str, scaler: Optional[torch.cuda.amp.GradScaler] = None
+    ) -> None:
+        super().__init__(precision, device, scaler)
+
+    def optimizer_step(
+        self,
+        model: Union["pl.LightningModule", torch.nn.Module],
+        optimizer: torch.optim.Optimizer,
+        optimizer_idx: int,
+        closure: Callable[[], Any],
+        **kwargs: Any,
+    ) -> None:
+        assert isinstance(
+            optimizer, MasterOptimizerWrapper
+        ), "MegatronHalfPrecisionPlugin supports only the optimizer with master parameters"
+
+        if self.scaler is None:
+            assert optimizer.fp32_grad_accumulation, "BF16 uses FP32 grad accumulation"
+            if optimizer.async_master_grads_allreudce:
+                # Execute the last step with asynchronous master gradients all-reduce
+                with optimizer.grad_sync():
+                    _ = closure()
+            else:
+                _ = closure()
+                optimizer.allreduce_main_grads()
+
+            self._after_closure(model, optimizer, optimizer_idx)
+            return optimizer.step(**kwargs)
+
+        if isinstance(optimizer, torch.optim.LBFGS):
+            raise MisconfigurationException(
+                f"Native AMP and the LBFGS optimizer are not compatible (optimizer {optimizer_idx})."
+            )
+        assert not optimizer.fp32_grad_accumulation, "FP16 uses FP16 grad accumulation"
+        closure_result = closure()
+
+        # TODO: Add an option for merged all-reduce
+
+        # cast fp16 grads to fp32 and copy to main grads, which are used for unscale and param update
+        optimizer.copy_model_grads_to_main_grads()
+        # `unscale` after the closure is executed but before the `on_before_optimizer_step` hook.
+        # unscale main (fp32) gradients
+        self.scaler.unscale_(optimizer)
+        self._after_closure(model, optimizer, optimizer_idx)
+        skipped_backward = closure_result is None
+        # in manual optimization, the closure does not return a value
+        if not isinstance(model, pl.LightningModule) or not model.automatic_optimization or not skipped_backward:
+            # note: the scaler will skip the `optimizer.step` if nonfinite gradients are found
+            self.scaler.step(optimizer, **kwargs)
+            self.scaler.update()
+            model.log('grad_scale', self.scaler.get_scale())
+
+    @contextmanager
+    def forward_context(self) -> Generator[None, None, None]:
+        """ No explicit precision casting. Inputs are supposed to be manually casted """
+        try:
+            yield
+        finally:
+            pass
