@@ -12,12 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from dataclasses import dataclass
-from typing import Any, Dict
-
 import torch
 from hydra.utils import instantiate
-from omegaconf import MISSING, DictConfig, OmegaConf, open_dict
+from omegaconf import DictConfig, open_dict
 from pytorch_lightning import Trainer
 from pytorch_lightning.loggers import LoggerCollection, TensorBoardLogger
 
@@ -28,6 +25,7 @@ from nemo.collections.tts.losses.aligner_loss import BinLoss, ForwardSumLoss
 from nemo.collections.tts.losses.fastpitchloss import DurationLoss, MelLoss, PitchLoss
 from nemo.collections.tts.models.base import SpectrogramGenerator
 from nemo.collections.tts.modules.fastpitch import FastPitchModule
+from nemo.collections.tts.torch.tts_data_types import SpeakerID
 from nemo.core.classes import Exportable
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.core.neural_types.elements import (
@@ -41,46 +39,50 @@ from nemo.core.neural_types.elements import (
     TokenLogDurationType,
 )
 from nemo.core.neural_types.neural_type import NeuralType
-from nemo.utils import logging
-
-
-@dataclass
-class FastPitchConfig:
-    parser: Dict[Any, Any] = MISSING
-    preprocessor: Dict[Any, Any] = MISSING
-    input_fft: Dict[Any, Any] = MISSING
-    output_fft: Dict[Any, Any] = MISSING
-    duration_predictor: Dict[Any, Any] = MISSING
-    pitch_predictor: Dict[Any, Any] = MISSING
+from nemo.utils import logging, model_utils
 
 
 class FastPitchModel(SpectrogramGenerator, Exportable):
     """FastPitch Model that is used to generate mel spectrograms from text"""
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
-        if isinstance(cfg, dict):
-            cfg = OmegaConf.create(cfg)
+        # Convert to Hydra 1.0 compatible DictConfig
+        cfg = model_utils.convert_model_config_to_dict_config(cfg)
+        cfg = model_utils.maybe_update_config_version(cfg)
+
+        # setup normalizer
+        self.normalizer = None
+        self.text_normalizer_call = None
+        self.text_normalizer_call_kwargs = {}
+        self._setup_normalizer(cfg)
+
+        self.learn_alignment = cfg.get("learn_alignment", False)
+
+        # setup vocabulary (=tokenizer) and input_fft_kwargs (supported only with self.learn_alignment=True)
+        input_fft_kwargs = {}
+        if self.learn_alignment:
+            self.vocab = None
+            self.ds_class_name = cfg.train_ds.dataset._target_.split(".")[-1]
+
+            if self.ds_class_name == "AudioToCharWithPriorAndPitchDataset":
+                self.vocab = AudioToCharWithDursF0Dataset.make_vocab(**cfg.train_ds.dataset.vocab)
+                input_fft_kwargs["n_embed"] = len(self.vocab.labels)
+                input_fft_kwargs["padding_idx"] = self.vocab.pad
+            elif self.ds_class_name == "TTSDataset":
+                self._setup_tokenizer(cfg)
+                assert self.vocab is not None
+                input_fft_kwargs["n_embed"] = len(self.vocab.tokens)
+                input_fft_kwargs["padding_idx"] = self.vocab.pad
+            else:
+                raise ValueError(f"Unknown dataset class: {self.ds_class_name}")
 
-        self.learn_alignment = False
-        if "learn_alignment" in cfg:
-            self.learn_alignment = cfg.learn_alignment
         self._parser = None
         self._tb_logger = None
         super().__init__(cfg=cfg, trainer=trainer)
 
-        schema = OmegaConf.structured(FastPitchConfig)
-        # ModelPT ensures that cfg is a DictConfig, but do this second check in case ModelPT changes
-        if isinstance(cfg, dict):
-            cfg = OmegaConf.create(cfg)
-        elif not isinstance(cfg, DictConfig):
-            raise ValueError(f"cfg was type: {type(cfg)}. Expected either a dict or a DictConfig")
-        # Ensure passed cfg is compliant with schema
-        OmegaConf.merge(cfg, schema)
-
-        self.bin_loss_warmup_epochs = 100
-        self.aligner = None
+        self.bin_loss_warmup_epochs = cfg.get("bin_loss_warmup_epochs", 100)
         self.log_train_images = False
-        self.mel_loss = MelLoss()
+
         loss_scale = 0.1 if self.learn_alignment else 1.0
         dur_loss_scale = loss_scale
         pitch_loss_scale = loss_scale
@@ -88,19 +90,18 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             dur_loss_scale = cfg.dur_loss_scale
         if "pitch_loss_scale" in cfg:
             pitch_loss_scale = cfg.pitch_loss_scale
+
+        self.mel_loss = MelLoss()
         self.pitch_loss = PitchLoss(loss_scale=pitch_loss_scale)
         self.duration_loss = DurationLoss(loss_scale=dur_loss_scale)
-        input_fft_kwargs = {}
+
+        self.aligner = None
         if self.learn_alignment:
             self.aligner = instantiate(self._cfg.alignment_module)
             self.forward_sum_loss = ForwardSumLoss()
             self.bin_loss = BinLoss()
-            self.vocab = AudioToCharWithDursF0Dataset.make_vocab(**self._cfg.train_ds.dataset.vocab)
-            input_fft_kwargs["n_embed"] = len(self.vocab.labels)
-            input_fft_kwargs["padding_idx"] = self.vocab.pad
 
         self.preprocessor = instantiate(self._cfg.preprocessor)
-
         input_fft = instantiate(self._cfg.input_fft, **input_fft_kwargs)
         output_fft = instantiate(self._cfg.output_fft)
         duration_predictor = instantiate(self._cfg.duration_predictor)
@@ -119,6 +120,39 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         )
         self._input_types = self._output_types = None
 
+    def _setup_normalizer(self, cfg):
+        if "text_normalizer" in cfg:
+            normalizer_kwargs = {}
+
+            if "whitelist" in cfg.text_normalizer:
+                normalizer_kwargs["whitelist"] = self.register_artifact(
+                    'text_normalizer.whitelist', cfg.text_normalizer.whitelist
+                )
+
+            self.normalizer = instantiate(cfg.text_normalizer, **normalizer_kwargs)
+            self.text_normalizer_call = self.normalizer.normalize
+            if "text_normalizer_call_kwargs" in cfg:
+                self.text_normalizer_call_kwargs = cfg.text_normalizer_call_kwargs
+
+    def _setup_tokenizer(self, cfg):
+        text_tokenizer_kwargs = {}
+        if "g2p" in cfg.text_tokenizer:
+            g2p_kwargs = {}
+
+            if "phoneme_dict" in cfg.text_tokenizer.g2p:
+                g2p_kwargs["phoneme_dict"] = self.register_artifact(
+                    'text_tokenizer.g2p.phoneme_dict', cfg.text_tokenizer.g2p.phoneme_dict,
+                )
+
+            if "heteronyms" in cfg.text_tokenizer.g2p:
+                g2p_kwargs["heteronyms"] = self.register_artifact(
+                    'text_tokenizer.g2p.heteronyms', cfg.text_tokenizer.g2p.heteronyms,
+                )
+
+            text_tokenizer_kwargs["g2p"] = instantiate(cfg.text_tokenizer.g2p, **g2p_kwargs)
+
+        self.vocab = instantiate(cfg.text_tokenizer, **text_tokenizer_kwargs)
+
     @property
     def tb_logger(self):
         if self._tb_logger is None:
@@ -139,10 +173,14 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             return self._parser
 
         if self.learn_alignment:
-            if self.vocab is None:
-                self.vocab = AudioToCharWithDursF0Dataset.make_vocab(**self._cfg.train_ds.dataset.vocab)
-            self._parser = self.vocab.encode
+            ds_class_name = self._cfg.train_ds.dataset._target_.split(".")[-1]
+
+            if ds_class_name == "AudioToCharWithPriorAndPitchDataset" or ds_class_name == "TTSDataset":
+                self._parser = self.vocab.encode
+            else:
+                raise ValueError(f"Unknown dataset class: {ds_class_name}")
         else:
+            # ds_class_name == "FastPitchDataset"
             self._parser = parsers.make_parser(
                 labels=self._cfg.labels,
                 name='en',
@@ -154,10 +192,13 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             )
         return self._parser
 
-    def parse(self, str_input: str) -> torch.tensor:
+    def parse(self, str_input: str, normalize=True) -> torch.tensor:
         if str_input[-1] not in [".", "!", "?"]:
             str_input = str_input + "."
 
+        if normalize and self.text_normalizer_call is not None:
+            str_input = self.text_normalizer_call(str_input, **self.text_normalizer_call_kwargs)
+
         tokens = self.parser(str_input)
 
         x = torch.tensor(tokens).unsqueeze_(0).long().to(self.device)
@@ -165,13 +206,13 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
 
     @typecheck(
         input_types={
-            "text": NeuralType(('B', 'T'), TokenIndex()),
-            "durs": NeuralType(('B', 'T'), TokenDurationType()),
-            "pitch": NeuralType(('B', 'T'), RegressionValuesType()),
+            "text": NeuralType(('B', 'T_text'), TokenIndex()),
+            "durs": NeuralType(('B', 'T_text'), TokenDurationType()),
+            "pitch": NeuralType(('B', 'T_audio'), RegressionValuesType()),
             "speaker": NeuralType(('B'), Index()),
             "pace": NeuralType(optional=True),
-            "spec": NeuralType(('B', 'D', 'T'), MelSpectrogramType(), optional=True),
-            "attn_prior": NeuralType(('B', 'T', 'T'), ProbsType(), optional=True),
+            "spec": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType(), optional=True),
+            "attn_prior": NeuralType(('B', 'T_spec', 'T_text'), ProbsType(), optional=True),
             "mel_lens": NeuralType(('B'), LengthsType(), optional=True),
             "input_lens": NeuralType(('B'), LengthsType(), optional=True),
         }
@@ -201,26 +242,37 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             input_lens=input_lens,
         )
 
-    @typecheck(output_types={"spect": NeuralType(('B', 'C', 'T'), MelSpectrogramType())})
+    @typecheck(output_types={"spect": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType())})
     def generate_spectrogram(self, tokens: 'torch.tensor', speaker: int = 0, pace: float = 1.0) -> torch.tensor:
         # FIXME: return masks as well?
         self.eval()
+        if isinstance(speaker, int):
+            speaker = torch.tensor([speaker]).to(self.device)
         spect, *_ = self(text=tokens, durs=None, pitch=None, speaker=speaker, pace=pace)
         return spect
 
     def training_step(self, batch, batch_idx):
-        attn_prior, durs = None, None
+        attn_prior, durs, speaker = None, None, None
         if self.learn_alignment:
-            audio, audio_lens, text, text_lens, attn_prior, pitch, speakers = batch
+            if self.ds_class_name == "AudioToCharWithPriorAndPitchDataset":
+                audio, audio_lens, text, text_lens, attn_prior, pitch, speaker = batch
+            elif self.ds_class_name == "TTSDataset":
+                if SpeakerID in self._train_dl.dataset.sup_data_types_set:
+                    audio, audio_lens, text, text_lens, attn_prior, pitch, _, speaker = batch
+                else:
+                    audio, audio_lens, text, text_lens, attn_prior, pitch, _ = batch
+            else:
+                raise ValueError(f"Unknown vocab class: {self.vocab.__class__.__name__}")
         else:
-            audio, audio_lens, text, text_lens, durs, pitch, speakers = batch
+            audio, audio_lens, text, text_lens, durs, pitch, speaker = batch
+
         mels, spec_len = self.preprocessor(input_signal=audio, length=audio_lens)
 
         mels_pred, _, _, log_durs_pred, pitch_pred, attn_soft, attn_logprob, attn_hard, attn_hard_dur, pitch = self(
             text=text,
             durs=durs,
             pitch=pitch,
-            speaker=speakers,
+            speaker=speaker,
             pace=1.0,
             spec=mels if self.learn_alignment else None,
             attn_prior=attn_prior,
@@ -251,7 +303,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             self.log("t_bin_loss", bin_loss)
 
         # Log images to tensorboard
-        if self.log_train_images:
+        if self.log_train_images and isinstance(self.logger, TensorBoardLogger):
             self.log_train_images = False
 
             self.tb_logger.add_image(
@@ -277,11 +329,20 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         return loss
 
     def validation_step(self, batch, batch_idx):
-        attn_prior, durs, speakers = None, None, None
+        attn_prior, durs, speaker = None, None, None
         if self.learn_alignment:
-            audio, audio_lens, text, text_lens, attn_prior, pitch, speakers = batch
+            if self.ds_class_name == "AudioToCharWithPriorAndPitchDataset":
+                audio, audio_lens, text, text_lens, attn_prior, pitch, speaker = batch
+            elif self.ds_class_name == "TTSDataset":
+                if SpeakerID in self._train_dl.dataset.sup_data_types_set:
+                    audio, audio_lens, text, text_lens, attn_prior, pitch, _, speaker = batch
+                else:
+                    audio, audio_lens, text, text_lens, attn_prior, pitch, _ = batch
+            else:
+                raise ValueError(f"Unknown vocab class: {self.vocab.__class__.__name__}")
         else:
-            audio, audio_lens, text, text_lens, durs, pitch, speakers = batch
+            audio, audio_lens, text, text_lens, durs, pitch, speaker = batch
+
         mels, mel_lens = self.preprocessor(input_signal=audio, length=audio_lens)
 
         # Calculate val loss on ground truth durations to better align L2 loss in time
@@ -289,7 +350,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             text=text,
             durs=durs,
             pitch=pitch,
-            speaker=speakers,
+            speaker=speaker,
             pace=1.0,
             spec=mels if self.learn_alignment else None,
             attn_prior=attn_prior,
@@ -325,17 +386,19 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         self.log("v_pitch_loss", pitch_loss)
 
         _, _, _, _, spec_target, spec_predict = outputs[0].values()
-        self.tb_logger.add_image(
-            "val_mel_target",
-            plot_spectrogram_to_numpy(spec_target[0].data.cpu().numpy()),
-            self.global_step,
-            dataformats="HWC",
-        )
-        spec_predict = spec_predict[0].data.cpu().numpy()
-        self.tb_logger.add_image(
-            "val_mel_predicted", plot_spectrogram_to_numpy(spec_predict), self.global_step, dataformats="HWC",
-        )
-        self.log_train_images = True
+
+        if isinstance(self.logger, TensorBoardLogger):
+            self.tb_logger.add_image(
+                "val_mel_target",
+                plot_spectrogram_to_numpy(spec_target[0].data.cpu().numpy()),
+                self.global_step,
+                dataformats="HWC",
+            )
+            spec_predict = spec_predict[0].data.cpu().numpy()
+            self.tb_logger.add_image(
+                "val_mel_predicted", plot_spectrogram_to_numpy(spec_predict), self.global_step, dataformats="HWC",
+            )
+            self.log_train_images = True
 
     def __setup_dataloader_from_config(self, cfg, shuffle_should_be: bool = True, name: str = "train"):
         if "dataset" not in cfg or not isinstance(cfg.dataset, DictConfig):
@@ -355,10 +418,27 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         elif not shuffle_should_be and cfg.dataloader_params.shuffle:
             logging.error(f"The {name} dataloader for {self} has shuffle set to True!!!")
 
-        kwargs_dict = {}
         if cfg.dataset._target_ == "nemo.collections.asr.data.audio_to_text.FastPitchDataset":
-            kwargs_dict["parser"] = self.parser
-        dataset = instantiate(cfg.dataset, **kwargs_dict)
+            logging.warning(
+                "FastPitchDataset will be deprecated in 1.8 version. "
+                "Please change your model to use config with Torch TTS Collection instead (e.g. see nemo.collections.tts.torch.data.TTSDataset)."
+            )
+            dataset = instantiate(cfg.dataset, parser=self.parser)
+        elif cfg.dataset._target_ == "nemo.collections.tts.torch.data.TTSDataset":
+            dataset = instantiate(
+                cfg.dataset,
+                text_normalizer=self.normalizer,
+                text_normalizer_call_kwargs=self.text_normalizer_call_kwargs,
+                text_tokenizer=self.vocab,
+            )
+        else:
+            if cfg.dataset._target_ == "nemo.collections.asr.data.audio_to_text.AudioToCharWithPriorAndPitchDataset":
+                logging.warning(
+                    "AudioToCharWithPriorAndPitchDataset will be deprecated in 1.8 version. "
+                    "Please change your model to use config with Torch TTS Collection instead (e.g. see nemo.collections.tts.torch.data.TTSDataset)."
+                )
+            dataset = instantiate(cfg.dataset)
+
         return torch.utils.data.DataLoader(dataset, collate_fn=dataset.collate_fn, **cfg.dataloader_params)
 
     def setup_training_data(self, cfg):
@@ -390,24 +470,25 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         return list_of_models
 
     ### Export code
-    def input_example(self):
+    def input_example(self, max_batch=1, max_dim=256):
         """
         Generates input examples for tracing etc.
         Returns:
             A tuple of input examples.
         """
         par = next(self.fastpitch.parameters())
+        sz = (max_batch, max_dim)
         inp = torch.randint(
-            0, self.fastpitch.encoder.word_emb.num_embeddings, (1, 44), device=par.device, dtype=torch.int64
+            0, self.fastpitch.encoder.word_emb.num_embeddings, sz, device=par.device, dtype=torch.int64
         )
-        pitch = torch.randn((1, 44), device=par.device, dtype=torch.float32) * 0.5
-        pace = torch.clamp((torch.randn((1, 44), device=par.device, dtype=torch.float32) + 1) * 0.1, min=0.01)
+        pitch = torch.randn(sz, device=par.device, dtype=torch.float32) * 0.5
+        pace = torch.clamp((torch.randn(sz, device=par.device, dtype=torch.float32) + 1) * 0.1, min=0.01)
 
         inputs = {'text': inp, 'pitch': pitch, 'pace': pace}
 
         if self.fastpitch.speaker_emb is not None:
             inputs['speaker'] = torch.randint(
-                0, self.fastpitch.speaker_emb.num_embeddings, (1,), device=par.device, dtype=torch.int64
+                0, self.fastpitch.speaker_emb.num_embeddings, (maz_batch,), device=par.device, dtype=torch.int64
             )
 
         return (inputs,)
@@ -428,17 +509,17 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
 
         # Define input_types and output_types as required by export()
         self._input_types = {
-            "text": NeuralType(('B', 'T'), TokenIndex()),
-            "pitch": NeuralType(('B', 'T'), RegressionValuesType()),
-            "pace": NeuralType(('B', 'T'), optional=True),
+            "text": NeuralType(('B', 'T_text'), TokenIndex()),
+            "pitch": NeuralType(('B', 'T_text'), RegressionValuesType()),
+            "pace": NeuralType(('B', 'T_text'), optional=True),
             "speaker": NeuralType(('B'), Index()),
         }
         self._output_types = {
-            "spect": NeuralType(('B', 'D', 'T'), MelSpectrogramType()),
+            "spect": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType()),
             "num_frames": NeuralType(('B'), TokenDurationType()),
-            "durs_predicted": NeuralType(('B', 'T'), TokenDurationType()),
-            "log_durs_predicted": NeuralType(('B', 'T'), TokenLogDurationType()),
-            "pitch_predicted": NeuralType(('B', 'T'), RegressionValuesType()),
+            "durs_predicted": NeuralType(('B', 'T_text'), TokenDurationType()),
+            "log_durs_predicted": NeuralType(('B', 'T_text'), TokenLogDurationType()),
+            "pitch_predicted": NeuralType(('B', 'T_text'), RegressionValuesType()),
         }
 
     def _export_teardown(self):
