@@ -24,20 +24,26 @@ from nemo_text_processing.text_normalization.normalize import Normalizer
 from tqdm import tqdm
 
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
-from nemo.collections.common.parts.patch_utils import stft_patch
-from nemo.collections.tts.torch.helpers import beta_binomial_prior_distribution, general_padding
+from nemo.collections.tts.torch.helpers import (
+    BetaBinomialInterpolator,
+    beta_binomial_prior_distribution,
+    general_padding,
+)
 from nemo.collections.tts.torch.tts_data_types import (
     DATA_STR2DATA_CLASS,
     MAIN_DATA_TYPES,
     VALID_SUPPLEMENTARY_DATA_TYPES,
-    DurationPrior,
+    AlignPriorMatrix,
     Durations,
     Energy,
+    LMTokens,
     LogMel,
     Pitch,
+    SpeakerID,
+    TTSDataType,
     WithLens,
 )
-from nemo.collections.tts.torch.tts_tokenizers import BaseTokenizer
+from nemo.collections.tts.torch.tts_tokenizers import BaseTokenizer, EnglishCharsTokenizer, EnglishPhonemesTokenizer
 from nemo.core.classes import Dataset
 from nemo.utils import logging
 
@@ -50,7 +56,7 @@ class TTSDataset(Dataset):
         text_tokenizer: Union[BaseTokenizer, Callable[[str], List[int]]],
         tokens: Optional[List[str]] = None,
         text_normalizer: Optional[Union[Normalizer, Callable[[str], str]]] = None,
-        text_normalizer_call_args: Optional[Dict] = None,
+        text_normalizer_call_kwargs: Optional[Dict] = None,
         text_tokenizer_pad_id: Optional[int] = None,
         sup_data_types: Optional[List[str]] = None,
         sup_data_path: Optional[Union[Path, str]] = None,
@@ -75,14 +81,15 @@ class TTSDataset(Dataset):
                 dataset. Each line in the .json file should be valid json. Note: the .json file itself is not valid
                 json. Each line should contain the following:
                     "audio_filepath": <PATH_TO_WAV>
+                    "text": <THE_TRANSCRIPT>,
+                    "normalized_text": <NORMALIZED_TRANSCRIPT> (Optional),
                     "mel_filepath": <PATH_TO_LOG_MEL_PT> (Optional)
                     "duration": <Duration of audio clip in seconds> (Optional)
-                    "text": <THE_TRANSCRIPT> (Optional)
             sample_rate (int): The sample rate of the audio. Or the sample rate that we will resample all files to.
             text_tokenizer (Optional[Union[BaseTokenizer, Callable[[str], List[int]]]]): BaseTokenizer or callable which represents text tokenizer.
             tokens (Optional[List[str]]): Tokens from text_tokenizer. Should be specified if text_tokenizer is not BaseTokenizer.
             text_normalizer (Optional[Union[Normalizer, Callable[[str], str]]]): Normalizer or callable which represents text normalizer.
-            text_normalizer_call_args (Optional[Dict]): Additional arguments for text_normalizer function.
+            text_normalizer_call_kwargs (Optional[Dict]): Additional arguments for text_normalizer function.
             text_tokenizer_pad_id (Optional[int]): Index of padding. Should be specified if text_tokenizer is not BaseTokenizer.
             sup_data_types (Optional[List[str]]): List of supplementary data types.
             sup_data_path (Optional[Union[Path, str]]): A folder that contains or will contain supplementary data (e.g. pitch).
@@ -105,12 +112,13 @@ class TTSDataset(Dataset):
             highfreq (Optional[int]): The highfreq input to the mel filter calculation. Defaults to None.
         Keyword Args:
             durs_file (Optional[str]): String path to pickled durations location.
-            durs_type (Optional[str]): Type of durations. Currently supported only "aligned-based".
+            durs_type (Optional[str]): Type of durations. Currently supported only "aligner-based".
+            use_beta_binomial_interpolator (Optional[bool]): Whether to use beta-binomial interpolator for calculating alignment prior matrix. Defaults to False.
             pitch_fmin (Optional[float]): The fmin input to librosa.pyin. Defaults to librosa.note_to_hz('C2').
             pitch_fmax (Optional[float]): The fmax input to librosa.pyin. Defaults to librosa.note_to_hz('C7').
-            pitch_avg (Optional[float]): The mean that we use to normalize the pitch.
+            pitch_mean (Optional[float]): The mean that we use to normalize the pitch.
             pitch_std (Optional[float]): The std that we use to normalize the pitch.
-            pitch_norm (Optional[bool]): Whether to normalize pitch (via pitch_avg and pitch_std) or not.
+            pitch_norm (Optional[bool]): Whether to normalize pitch (via pitch_mean and pitch_std) or not.
         """
         super().__init__()
 
@@ -118,7 +126,9 @@ class TTSDataset(Dataset):
         self.text_normalizer_call = (
             self.text_normalizer.normalize if isinstance(self.text_normalizer, Normalizer) else self.text_normalizer
         )
-        self.text_normalizer_call_args = text_normalizer_call_args
+        self.text_normalizer_call_kwargs = (
+            text_normalizer_call_kwargs if text_normalizer_call_kwargs is not None else {}
+        )
 
         self.text_tokenizer = text_tokenizer
 
@@ -159,20 +169,23 @@ class TTSDataset(Dataset):
 
                     file_info = {
                         "audio_filepath": item["audio_filepath"],
+                        "original_text": item["text"],
                         "mel_filepath": item["mel_filepath"] if "mel_filepath" in item else None,
                         "duration": item["duration"] if "duration" in item else None,
-                        "text_tokens": None,
+                        "speaker_id": item["speaker"] if "speaker" in item else None,
                     }
 
-                    if "text" in item:
+                    if "normalized_text" not in item:
                         text = item["text"]
 
                         if self.text_normalizer is not None:
-                            text = self.text_normalizer_call(text, **self.text_normalizer_call_args)
+                            text = self.text_normalizer_call(text, **self.text_normalizer_call_kwargs)
 
-                        text_tokens = self.text_tokenizer(text)
-                        file_info["raw_text"] = item["text"]
-                        file_info["text_tokens"] = text_tokens
+                        file_info["normalized_text"] = text
+                        file_info["text_tokens"] = self.text_tokenizer(text)
+                    else:
+                        file_info["normalized_text"] = item["normalized_text"]
+                        file_info["text_tokens"] = self.text_tokenizer(item["normalized_text"])
 
                     audio_files.append(file_info)
 
@@ -251,7 +264,7 @@ class TTSDataset(Dataset):
             'none': None,
         }.get(self.window, None)
 
-        self.stft = lambda x: stft_patch(
+        self.stft = lambda x: torch.stft(
             input=x,
             n_fft=self.n_fft,
             hop_length=self.hop_len,
@@ -281,22 +294,28 @@ class TTSDataset(Dataset):
                 self.durs.append(durs)
             else:
                 raise NotImplementedError(
-                    f"{durs_type} duration type is not supported. Only align-based is supported at this moment."
+                    f"{durs_type} duration type is not supported. Only aligner-based is supported at this moment."
                 )
 
-    def add_duration_prior(self, **kwargs):
-        pass
+    def add_align_prior_matrix(self, **kwargs):
+        self.use_beta_binomial_interpolator = kwargs.pop('use_beta_binomial_interpolator', False)
+
+        if self.use_beta_binomial_interpolator:
+            self.beta_binomial_interpolator = BetaBinomialInterpolator()
 
     def add_pitch(self, **kwargs):
         self.pitch_fmin = kwargs.pop("pitch_fmin", librosa.note_to_hz('C2'))
         self.pitch_fmax = kwargs.pop("pitch_fmax", librosa.note_to_hz('C7'))
-        self.pitch_avg = kwargs.pop("pitch_avg", None)
+        self.pitch_mean = kwargs.pop("pitch_mean", None)
         self.pitch_std = kwargs.pop("pitch_std", None)
         self.pitch_norm = kwargs.pop("pitch_norm", False)
 
     def add_energy(self, **kwargs):
         pass
 
+    def add_speaker_id(self, **kwargs):
+        pass
+
     def get_spec(self, audio):
         with torch.cuda.amp.autocast(enabled=False):
             spec = self.stft(audio)
@@ -344,17 +363,21 @@ class TTSDataset(Dataset):
         if Durations in self.sup_data_types_set:
             durations = self.durs[index]
 
-        duration_prior = None
-        if DurationPrior in self.sup_data_types_set:
-            prior_path = Path(self.sup_data_path) / f"pr_{audio_stem}.pt"
-
-            if prior_path.exists():
-                duration_prior = torch.load(prior_path)
+        align_prior_matrix = None
+        if AlignPriorMatrix in self.sup_data_types_set:
+            if self.use_beta_binomial_interpolator:
+                mel_len = self.get_log_mel(audio).shape[2]
+                align_prior_matrix = torch.from_numpy(self.beta_binomial_interpolator(mel_len, text_length.item()))
             else:
-                log_mel_length = torch.tensor(self.get_log_mel(audio).squeeze(0).shape[1]).long()
-                duration_prior = beta_binomial_prior_distribution(text_length, log_mel_length)
-                duration_prior = torch.from_numpy(duration_prior)
-                torch.save(duration_prior, prior_path)
+                prior_path = Path(self.sup_data_path) / f"pr_{audio_stem}.pt"
+
+                if prior_path.exists():
+                    align_prior_matrix = torch.load(prior_path)
+                else:
+                    mel_len = self.get_log_mel(audio).shape[2]
+                    align_prior_matrix = beta_binomial_prior_distribution(text_length, mel_len)
+                    align_prior_matrix = torch.from_numpy(align_prior_matrix)
+                    torch.save(align_prior_matrix, prior_path)
 
         pitch, pitch_length = None, None
         if Pitch in self.sup_data_types_set:
@@ -379,9 +402,9 @@ class TTSDataset(Dataset):
                 pitch = torch.from_numpy(pitch).float()
                 torch.save(pitch, pitch_path)
 
-            if self.pitch_avg is not None and self.pitch_std is not None and self.pitch_norm:
-                pitch -= self.pitch_avg
-                pitch[pitch == -self.pitch_avg] = 0.0  # Zero out values that were perviously zero
+            if self.pitch_mean is not None and self.pitch_std is not None and self.pitch_norm:
+                pitch -= self.pitch_mean
+                pitch[pitch == -self.pitch_mean] = 0.0  # Zero out values that were perviously zero
                 pitch /= self.pitch_std
 
             pitch_length = torch.tensor(len(pitch)).long()
@@ -398,6 +421,10 @@ class TTSDataset(Dataset):
 
             energy_length = torch.tensor(len(energy)).long()
 
+        speaker_id = None
+        if SpeakerID in self.sup_data_types_set:
+            speaker_id = torch.tensor(sample["speaker_id"]).long()
+
         return (
             audio,
             audio_length,
@@ -406,11 +433,12 @@ class TTSDataset(Dataset):
             log_mel,
             log_mel_length,
             durations,
-            duration_prior,
+            align_prior_matrix,
             pitch,
             pitch_length,
             energy,
             energy_length,
+            speaker_id,
         )
 
     def __len__(self):
@@ -421,7 +449,7 @@ class TTSDataset(Dataset):
         for data_type in MAIN_DATA_TYPES + self.sup_data_types:
             result.append(data_dict[data_type.name])
 
-            if issubclass(data_type, WithLens):
+            if issubclass(data_type, TTSDataType) and issubclass(data_type, WithLens):
                 result.append(data_dict[f"{data_type.name}_lens"])
 
         return tuple(result)
@@ -435,11 +463,12 @@ class TTSDataset(Dataset):
             _,
             log_mel_lengths,
             durations_list,
-            duration_priors_list,
+            align_prior_matrices_list,
             pitches,
             pitches_lengths,
             energies,
             energies_lengths,
+            _,
         ) = zip(*batch)
 
         max_audio_len = max(audio_lengths).item()
@@ -452,16 +481,16 @@ class TTSDataset(Dataset):
         if LogMel in self.sup_data_types_set:
             log_mel_pad = torch.finfo(batch[0][2].dtype).tiny
 
-        duration_priors = (
+        align_prior_matrices = (
             torch.zeros(
-                len(duration_priors_list),
-                max([prior_i.shape[0] for prior_i in duration_priors_list]),
-                max([prior_i.shape[1] for prior_i in duration_priors_list]),
+                len(align_prior_matrices_list),
+                max([prior_i.shape[0] for prior_i in align_prior_matrices_list]),
+                max([prior_i.shape[1] for prior_i in align_prior_matrices_list]),
             )
-            if DurationPrior in self.sup_data_types_set
+            if AlignPriorMatrix in self.sup_data_types_set
             else []
         )
-        audios, tokens, log_mels, durations_list, pitches, energies = [], [], [], [], [], []
+        audios, tokens, log_mels, durations_list, pitches, energies, speaker_ids = [], [], [], [], [], [], []
 
         for i, sample_tuple in enumerate(batch):
             (
@@ -472,11 +501,12 @@ class TTSDataset(Dataset):
                 log_mel,
                 log_mel_len,
                 durations,
-                duration_prior,
+                align_prior_matrix,
                 pitch,
                 pitch_length,
                 energy,
                 energy_length,
+                speaker_id,
             ) = sample_tuple
 
             audio = general_padding(audio, audio_len.item(), max_audio_len)
@@ -489,12 +519,16 @@ class TTSDataset(Dataset):
                 log_mels.append(general_padding(log_mel, log_mel_len, max_log_mel_len, pad_value=log_mel_pad))
             if Durations in self.sup_data_types_set:
                 durations_list.append(general_padding(durations, len(durations), max_durations_len))
-            if DurationPrior in self.sup_data_types_set:
-                duration_priors[i, : duration_prior.shape[0], : duration_prior.shape[1]] = duration_prior
+            if AlignPriorMatrix in self.sup_data_types_set:
+                align_prior_matrices[
+                    i, : align_prior_matrix.shape[0], : align_prior_matrix.shape[1]
+                ] = align_prior_matrix
             if Pitch in self.sup_data_types_set:
                 pitches.append(general_padding(pitch, pitch_length.item(), max_pitches_len))
             if Energy in self.sup_data_types_set:
                 energies.append(general_padding(energy, energy_length.item(), max_energies_len))
+            if SpeakerID in self.sup_data_types_set:
+                speaker_ids.append(speaker_id)
 
         data_dict = {
             "audio": torch.stack(audios),
@@ -504,11 +538,12 @@ class TTSDataset(Dataset):
             "log_mel": torch.stack(log_mels) if LogMel in self.sup_data_types_set else None,
             "log_mel_lens": torch.stack(log_mel_lengths) if LogMel in self.sup_data_types_set else None,
             "durations": torch.stack(durations_list) if Durations in self.sup_data_types_set else None,
-            "duration_prior": duration_priors if DurationPrior in self.sup_data_types_set else None,
+            "align_prior_matrix": align_prior_matrices if AlignPriorMatrix in self.sup_data_types_set else None,
             "pitch": torch.stack(pitches) if Pitch in self.sup_data_types_set else None,
             "pitch_lens": torch.stack(pitches_lengths) if Pitch in self.sup_data_types_set else None,
             "energy": torch.stack(energies) if Energy in self.sup_data_types_set else None,
             "energy_lens": torch.stack(energies_lengths) if Energy in self.sup_data_types_set else None,
+            "speaker_id": torch.stack(speaker_ids) if SpeakerID in self.sup_data_types_set else None,
         }
 
         return data_dict
@@ -517,3 +552,97 @@ class TTSDataset(Dataset):
         data_dict = self.general_collate_fn(batch)
         joined_data = self.join_data(data_dict)
         return joined_data
+
+
+class MixerTTSDataset(TTSDataset):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def _albert(self):
+        from transformers import AlbertTokenizer  # noqa pylint: disable=import-outside-toplevel
+
+        self.lm_model_tokenizer = AlbertTokenizer.from_pretrained('albert-base-v2')
+        self.lm_padding_value = self.lm_model_tokenizer._convert_token_to_id('<pad>')
+        space_value = self.lm_model_tokenizer._convert_token_to_id('▁')
+
+        self.id2lm_tokens = {}
+        for i, d in enumerate(self.data):
+            normalized_text = d["normalized_text"]
+
+            assert isinstance(self.text_tokenizer, EnglishPhonemesTokenizer) or isinstance(
+                self.text_tokenizer, EnglishCharsTokenizer
+            )
+            preprocess_text_as_tts_input = self.text_tokenizer.text_preprocessing_func(normalized_text)
+
+            lm_tokens_as_ids = self.lm_model_tokenizer.encode(preprocess_text_as_tts_input, add_special_tokens=False)
+
+            if self.text_tokenizer.pad_with_space:
+                lm_tokens_as_ids = [space_value] + lm_tokens_as_ids + [space_value]
+
+            self.id2lm_tokens[i] = lm_tokens_as_ids
+
+    def add_lm_tokens(self, **kwargs):
+        lm_model = kwargs.pop('lm_model')
+
+        if lm_model == "albert":
+            self._albert()
+        else:
+            raise NotImplementedError(
+                f"{lm_model} lm model is not supported. Only albert is supported at this moment."
+            )
+
+    def __getitem__(self, index):
+        (
+            audio,
+            audio_length,
+            text,
+            text_length,
+            log_mel,
+            log_mel_length,
+            durations,
+            align_prior_matrix,
+            pitch,
+            pitch_length,
+            energy,
+            energy_length,
+            speaker_id,
+        ) = super().__getitem__(index)
+
+        lm_tokens = None
+        if LMTokens in self.sup_data_types_set:
+            lm_tokens = torch.tensor(self.id2lm_tokens[index]).long()
+
+        return (
+            audio,
+            audio_length,
+            text,
+            text_length,
+            log_mel,
+            log_mel_length,
+            durations,
+            align_prior_matrix,
+            pitch,
+            pitch_length,
+            energy,
+            energy_length,
+            speaker_id,
+            lm_tokens,
+        )
+
+    def _collate_fn(self, batch):
+        batch = list(zip(*batch))
+        data_dict = self.general_collate_fn(list(zip(*batch[:13])))
+        lm_tokens_list = batch[13]
+
+        if LMTokens in self.sup_data_types_set:
+            lm_tokens = torch.full(
+                (len(lm_tokens_list), max([lm_tokens.shape[0] for lm_tokens in lm_tokens_list])),
+                fill_value=self.lm_padding_value,
+            )
+            for i, lm_tokens_i in enumerate(lm_tokens_list):
+                lm_tokens[i, : lm_tokens_i.shape[0]] = lm_tokens_i
+
+            data_dict[LMTokens.name] = lm_tokens
+
+        joined_data = self.join_data(data_dict)
+        return joined_data
