@@ -47,7 +47,7 @@ class SaveRestoreConnector:
         Args:
             model: ModelPT object to be saved.
             save_path: Path to .nemo file where model instance should be saved
-		"""
+        """
 
         if is_global_rank_zero():
             with tempfile.TemporaryDirectory() as tmpdir:
@@ -74,27 +74,27 @@ class SaveRestoreConnector:
         trainer: Trainer = None,
     ):
         """
-		Restores model instance (weights and configuration) into .nemo file
+        Restores model instance (weights and configuration) into .nemo file
 
-		Args:
-		restore_path: path to .nemo file from which model should be instantiated
-		override_config_path: path to a yaml config that will override the internal
-			config file or an OmegaConf / DictConfig object representing the model config.
-		map_location: Optional torch.device() to map the instantiated model to a device.
-			By default (None), it will select a GPU if available, falling back to CPU otherwise.
-		strict: Passed to load_state_dict. By default True
-		return_config: If set to true, will return just the underlying config of the restored
-			model as an OmegaConf DictConfig object without instantiating the model.
+        Args:
+            restore_path: path to .nemo file from which model should be instantiated
+            override_config_path: path to a yaml config that will override the internal
+                config file or an OmegaConf / DictConfig object representing the model config.
+            map_location: Optional torch.device() to map the instantiated model to a device.
+                By default (None), it will select a GPU if available, falling back to CPU otherwise.
+            strict: Passed to load_state_dict. By default True
+            return_config: If set to true, will return just the underlying config of the restored
+                model as an OmegaConf DictConfig object without instantiating the model.
 
-		Example:
-			```
-			model = nemo.collections.asr.models.EncDecCTCModel.restore_from('asr.nemo')
-			assert isinstance(model, nemo.collections.asr.models.EncDecCTCModel)
-			```
+        Example:
+            ```
+            model = nemo.collections.asr.models.EncDecCTCModel.restore_from('asr.nemo')
+            assert isinstance(model, nemo.collections.asr.models.EncDecCTCModel)
+            ```
 
-		Returns:
-		An instance of type cls or its underlying config (if return_config is set).
-		"""
+        Returns:
+            An instance of type cls or its underlying config (if return_config is set).
+        """
         # Get path where the command is executed - the artifacts will be "retrieved" there
         # (original .nemo behavior)
         cwd = os.getcwd()
@@ -142,9 +142,14 @@ class SaveRestoreConnector:
                 instance = calling_cls.from_config_dict(config=conf, trainer=trainer)
                 instance = instance.to(map_location)
                 # add load_state_dict override
-                instance.load_state_dict(
-                    self._load_state_dict_from_disk(model_weights, map_location=map_location), strict=strict
-                )
+                state_dict = self._load_state_dict_from_disk(model_weights, map_location=map_location)
+                if conf.get('megatron_amp_O2', False):
+                    new_state_dict = {}
+                    for key in state_dict.keys():
+                        new_key = key.replace('model.', 'model.module.', 1)
+                        new_state_dict[new_key] = state_dict[key]
+                    state_dict = new_state_dict
+                instance.load_state_dict(state_dict, strict=strict)
 
                 logging.info(f'Model {instance.__class__.__name__} was successfully restored from {restore_path}.')
                 instance._set_model_restore_state(is_being_restored=False)
@@ -228,31 +233,33 @@ class SaveRestoreConnector:
         return state_dict
 
     def register_artifact(self, model, config_path: str, src: str, verify_src_exists: bool = True):
-        """ Register model artifacts with this function. These artifacts (files) will be included inside .nemo file
-            when model.save_to("mymodel.nemo") is called.        
+        """
+        Register model artifacts with this function. These artifacts (files) will be included inside .nemo file
+        when model.save_to("mymodel.nemo") is called.
 
-            How it works:
-            1. It always returns existing absolute path which can be used during Model constructor call
-                EXCEPTION: src is None or "" in which case nothing will be done and src will be returned
-            2. It will add (config_path, model_utils.ArtifactItem()) pair to self.artifacts
+        How it works:
+        1. It always returns existing absolute path which can be used during Model constructor call
+            EXCEPTION: src is None or "" in which case nothing will be done and src will be returned
+        2. It will add (config_path, model_utils.ArtifactItem()) pair to self.artifacts
 
-            If "src" is local existing path, then it will be returned in absolute path form.
-            elif "src" starts with "nemo_file:unique_artifact_name":
-                .nemo will be untarred to a temporary folder location and an actual existing path will be returned
-            else an error will be raised.
+        If "src" is local existing path, then it will be returned in absolute path form.
+        elif "src" starts with "nemo_file:unique_artifact_name":
+            .nemo will be untarred to a temporary folder location and an actual existing path will be returned
+        else an error will be raised.
 
-            WARNING: use .register_artifact calls in your models' constructors.
-            The returned path is not guaranteed to exist after you have exited your model's constuctor.
+        WARNING: use .register_artifact calls in your models' constructors.
+        The returned path is not guaranteed to exist after you have exited your model's constructor.
 
-            Args:
-                model: ModelPT object to register artifact for.
-                config_path (str): Artifact key. Usually corresponds to the model config.
-                src (str): Path to artifact.
-                verify_src_exists (bool): If set to False, then the artifact is optional and register_artifact will return None even if 
-                                          src is not found. Defaults to True.
+        Args:
+            model: ModelPT object to register artifact for.
+            config_path (str): Artifact key. Usually corresponds to the model config.
+            src (str): Path to artifact.
+            verify_src_exists (bool): If set to False, then the artifact is optional and register_artifact will return
+                None even if src is not found. Defaults to True.
 
-            Returns:
-                str: If src is not None or empty it always returns absolute path which is guaranteed to exists during model instnce life
+        Returns:
+            str: If src is not None or empty it always returns absolute path which is guaranteed to exists during model
+                instance life
         """
         app_state = AppState()
 
@@ -365,7 +372,7 @@ class SaveRestoreConnector:
                     OmegaConf.update(conf, conf_path, item.path)
                 else:
                     OmegaConf.update(conf, conf_path, item.hashed_path)
-            with open(path2yaml_file, 'w') as fout:
+            with open(path2yaml_file, 'w', encoding='utf-8') as fout:
                 OmegaConf.save(config=conf, f=fout, resolve=True)
 
     def _inject_model_parallel_rank_for_ckpt(self, dirname, basename):
