@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from nemo.core.optim.adafactor import Adafactor
 from nemo.core.optim.lr_scheduler import (
     CosineAnnealing,
     InverseSquareRootAnnealing,
@@ -20,10 +21,12 @@ from nemo.core.optim.lr_scheduler import (
     PolynomialHoldDecayAnnealing,
     SquareAnnealing,
     SquareRootAnnealing,
+    T5InverseSquareRootAnnealing,
     WarmupAnnealing,
     WarmupHoldPolicy,
     WarmupPolicy,
     prepare_lr_scheduler,
 )
 from nemo.core.optim.novograd import Novograd
+from nemo.core.optim.optimizer_with_master_params import MasterOptimizerWrapper
 from nemo.core.optim.optimizers import get_optimizer, parse_optimizer_args, register_optimizer
