@@ -13,8 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo_text_processing.text_normalization.en.graph_utils import GraphFst, delete_extra_space, delete_space
-from nemo_text_processing.text_normalization.en.utils import get_abs_path
+from nemo_text_processing.text_normalization.en.graph_utils import NEMO_SIGMA, GraphFst, delete_extra_space
 
 try:
     import pynini
@@ -69,16 +68,8 @@ class DecimalFst(GraphFst):
             cardinal.graph_hundred_component_at_least_one_none_zero_digit
         )
 
-        graph_decimal = pynini.string_file(get_abs_path("data/numbers/digit.tsv"))
-        graph_decimal |= pynini.string_file(get_abs_path("data/numbers/zero.tsv"))
+        self.graph = cardinal.single_digits_graph.optimize()
 
-        graph_decimal = (
-            pynini.cross("zero", "0")
-            | graph_decimal
-            | (graph_decimal | pynini.cross("o", "0"))
-            + pynini.closure(delete_space + (graph_decimal | pynini.cross("o", "0")), 1)
-        )
-        self.graph = pynini.invert(graph_decimal).optimize()
         if not deterministic:
             self.graph = self.graph | cardinal_graph
 
@@ -98,6 +89,26 @@ class DecimalFst(GraphFst):
             final_graph_wo_sign, cardinal_graph_hundred_component_at_least_one_none_zero_digit
         )
 
+        # reduce options for non_deterministic and allow either "oh" or "zero", but not combination
+        if not deterministic:
+            no_oh_zero = pynini.difference(
+                NEMO_SIGMA,
+                (NEMO_SIGMA + "oh" + NEMO_SIGMA + "zero" + NEMO_SIGMA)
+                | (NEMO_SIGMA + "zero" + NEMO_SIGMA + "oh" + NEMO_SIGMA),
+            ).optimize()
+            no_zero_oh = pynini.difference(
+                NEMO_SIGMA, NEMO_SIGMA + pynini.accep("zero") + NEMO_SIGMA + pynini.accep("oh") + NEMO_SIGMA
+            ).optimize()
+
+            self.final_graph_wo_negative |= pynini.compose(
+                self.final_graph_wo_negative,
+                pynini.cdrewrite(
+                    pynini.cross("integer_part: \"zero\"", "integer_part: \"oh\""), NEMO_SIGMA, NEMO_SIGMA, NEMO_SIGMA
+                ),
+            )
+            self.final_graph_wo_negative = pynini.compose(self.final_graph_wo_negative, no_oh_zero).optimize()
+            self.final_graph_wo_negative = pynini.compose(self.final_graph_wo_negative, no_zero_oh).optimize()
+
         final_graph = optional_graph_negative + self.final_graph_wo_negative
 
         final_graph = self.add_tokens(final_graph)
