@@ -19,7 +19,6 @@ from argparse import ArgumentParser
 from typing import List, Tuple
 
 from joblib import Parallel, delayed
-from nemo_text_processing.text_normalization.data_loader_utils import post_process_punctuation, pre_process
 from nemo_text_processing.text_normalization.normalize import Normalizer
 from tqdm import tqdm
 
@@ -40,12 +39,20 @@ try:
 except (ModuleNotFoundError, ImportError):
     PYNINI_AVAILABLE = False
 
+try:
+    from nemo.collections.nlp.data.text_normalization.utils import post_process_punct
+    from nemo_text_processing.text_normalization.data_loader_utils import pre_process
+
+    NLP_AVAILABLE = True
+except (ModuleNotFoundError, ImportError):
+    NLP_AVAILABLE = False
+
 """
 The script provides multiple normalization options and chooses the best one that minimizes CER of the ASR output
 (most of the semiotic classes use deterministic=False flag).
 
 To run this script with a .json manifest file, the manifest file should contain the following fields:
-    "audio_filepath" - path to the audio file
+    "audio_data" - path to the audio file
     "text" - raw text
     "pred_text" - ASR model prediction
     
@@ -103,14 +110,7 @@ class NormalizerWithAudio(Normalizer):
             whitelist=whitelist,
         )
 
-    def normalize(
-        self,
-        text: str,
-        n_tagged: int,
-        punct_pre_process: bool = True,
-        punct_post_process: bool = True,
-        verbose: bool = False,
-    ) -> str:
+    def normalize(self, text: str, n_tagged: int, punct_post_process: bool = True, verbose: bool = False,) -> str:
         """
         Main function. Normalizes tokens from written to spoken form
             e.g. 12 kg -> twelve kilograms
@@ -118,57 +118,75 @@ class NormalizerWithAudio(Normalizer):
         Args:
             text: string that may include semiotic classes
             n_tagged: number of tagged options to consider, -1 - to get all possible tagged options
-            punct_pre_process: whether to perform punctuation pre-processing, for example, [25] -> [ 25 ]
             punct_post_process: whether to normalize punctuation
             verbose: whether to print intermediate meta information
 
         Returns:
             normalized text options (usually there are multiple ways of normalizing a given semiotic class)
         """
-        if punct_pre_process:
+        original_text = text
+
+        if self.lang == "en":
             text = pre_process(text)
         text = text.strip()
         if not text:
             if verbose:
                 print(text)
             return text
-
         text = pynini.escape(text)
 
         if n_tagged == -1:
-            tagged_texts = rewrite.rewrites(text, self.tagger.fst)
+            if self.lang == "en":
+                try:
+                    tagged_texts = rewrite.rewrites(text, self.tagger.fst_no_digits)
+                except pynini.lib.rewrite.Error:
+                    tagged_texts = rewrite.rewrites(text, self.tagger.fst)
+            else:
+                tagged_texts = rewrite.rewrites(text, self.tagger.fst)
         else:
-            tagged_texts = rewrite.top_rewrites(text, self.tagger.fst, nshortest=n_tagged)
+            if self.lang == "en":
+                try:
+                    tagged_texts = rewrite.top_rewrites(text, self.tagger.fst_no_digits, nshortest=n_tagged)
+                except pynini.lib.rewrite.Error:
+                    tagged_texts = rewrite.top_rewrites(text, self.tagger.fst, nshortest=n_tagged)
+            else:
+                tagged_texts = rewrite.top_rewrites(text, self.tagger.fst, nshortest=n_tagged)
+
         # non-deterministic Eng normalization uses tagger composed with verbalizer, no permutation in between
-        if self.lang == 'en':
+        if self.lang == "en":
             normalized_texts = tagged_texts
         else:
             normalized_texts = []
             for tagged_text in tagged_texts:
-                self._verbalize(tagged_text, normalized_texts)
+                self._verbalize(tagged_text, normalized_texts, verbose=verbose)
 
         if len(normalized_texts) == 0:
             raise ValueError()
-        if punct_post_process:
-            normalized_texts = [post_process_punctuation(t) for t in normalized_texts]
 
+        if punct_post_process:
             # do post-processing based on Moses detokenizer
             if self.processor:
                 normalized_texts = [self.processor.detokenize([t]) for t in normalized_texts]
+                normalized_texts = [
+                    post_process_punct(input=original_text, normalized_text=t) for t in normalized_texts
+                ]
+            else:
+                print("NEMO_NLP collection is not available: skipping punctuation post_processing")
+
         normalized_texts = set(normalized_texts)
         return normalized_texts
 
-    def _verbalize(self, tagged_text: str, normalized_texts: List[str]):
+    def _verbalize(self, tagged_text: str, normalized_texts: List[str], verbose: bool = False):
         """
         Verbalizes tagged text
 
         Args:
             tagged_text: text with tags
             normalized_texts: list of possible normalization options
+            verbose: if true prints intermediate classification results
         """
 
         def get_verbalized_text(tagged_text):
-            tagged_text = pynini.escape(tagged_text)
             return rewrite.rewrites(tagged_text, self.verbalizer.fst)
 
         self.parser(tagged_text)
@@ -176,18 +194,28 @@ class NormalizerWithAudio(Normalizer):
         tags_reordered = self.generate_permutations(tokens)
         for tagged_text_reordered in tags_reordered:
             try:
+                tagged_text_reordered = pynini.escape(tagged_text_reordered)
                 normalized_texts.extend(get_verbalized_text(tagged_text_reordered))
+                if verbose:
+                    print(tagged_text_reordered)
+
             except pynini.lib.rewrite.Error:
                 continue
 
     def select_best_match(
-        self, normalized_texts: List[str], pred_text: str, verbose: bool = False, remove_punct: bool = False
+        self,
+        normalized_texts: List[str],
+        input_text: str,
+        pred_text: str,
+        verbose: bool = False,
+        remove_punct: bool = False,
     ):
         """
         Selects the best normalization option based on the lowest CER
 
         Args:
             normalized_texts: normalized text options
+            input_text: input text
             pred_text: ASR model transcript of the audio file corresponding to the normalized text
             verbose: whether to print intermediate meta information
             remove_punct: whether to remove punctuation before calculating CER
@@ -195,6 +223,8 @@ class NormalizerWithAudio(Normalizer):
         Returns:
             normalized text with the lowest CER and CER value
         """
+        if pred_text == "":
+            return input_text, 1000
         normalized_texts = calculate_cer(normalized_texts, pred_text, remove_punct)
         normalized_texts = sorted(normalized_texts, key=lambda x: x[1])
         normalized_text, cer = normalized_texts[0]
@@ -252,7 +282,9 @@ def parse_args():
     parser.add_argument(
         "--input_case", help="input capitalization", choices=["lower_cased", "cased"], default="cased", type=str
     )
-    parser.add_argument("--language", help="Select target language", choices=["en", "ru"], default="en", type=str)
+    parser.add_argument(
+        "--language", help="Select target language", choices=["en", "ru", "de"], default="en", type=str
+    )
     parser.add_argument("--audio_data", default=None, help="path to an audio file or .json manifest")
     parser.add_argument(
         '--model', type=str, default='QuartzNet15x5Base-En', help='Pre-trained model name or path to model checkpoint'
@@ -260,14 +292,11 @@ def parse_args():
     parser.add_argument(
         "--n_tagged",
         type=int,
-        default=10000,
+        default=30,
         help="number of tagged options to consider, -1 - return all possible tagged options",
     )
     parser.add_argument("--verbose", help="print info for debugging", action="store_true")
     parser.add_argument("--remove_punct", help="remove punctuation before calculating cer", action="store_true")
-    parser.add_argument(
-        "--no_punct_pre_process", help="set to True to disable punctuation pre processing", action="store_true"
-    )
     parser.add_argument(
         "--no_punct_post_process", help="set to True to disable punctuation post processing", action="store_true"
     )
@@ -285,18 +314,23 @@ def parse_args():
 
 def _normalize_line(normalizer: NormalizerWithAudio, line: str):
     line = json.loads(line)
-    pred_text = line['pred_text']
+    pred_text = line["pred_text"]
 
     normalized_texts = normalizer.normalize(
-        text=line['text'],
+        text=line["text"],
         verbose=args.verbose,
         n_tagged=args.n_tagged,
-        punct_pre_process=not args.no_punct_pre_process,
         punct_post_process=not args.no_punct_post_process,
     )
-    normalized_text, cer = normalizer.select_best_match(normalized_texts, pred_text, args.verbose, args.remove_punct)
-    line['nemo_normalized'] = normalized_text
-    line['CER_nemo_normalized'] = cer
+    normalized_text, cer = normalizer.select_best_match(
+        normalized_texts=normalized_texts,
+        input_text=line["text"],
+        pred_text=pred_text,
+        verbose=args.verbose,
+        remove_punct=args.remove_punct,
+    )
+    line["nemo_normalized"] = normalized_text
+    line["CER_nemo_normalized"] = cer
     return line
 
 
@@ -337,7 +371,7 @@ if __name__ == "__main__":
         raise ValueError("NeMo ASR collection is not installed.")
     start = time.time()
     args.whitelist = os.path.abspath(args.whitelist) if args.whitelist else None
-    if args.text:
+    if args.text is not None:
         normalizer = NormalizerWithAudio(
             input_case=args.input_case,
             lang=args.language,
@@ -353,23 +387,27 @@ if __name__ == "__main__":
             text=args.text,
             verbose=args.verbose,
             n_tagged=args.n_tagged,
-            punct_pre_process=not args.no_punct_pre_process,
             punct_post_process=not args.no_punct_post_process,
         )
+
         if args.audio_data:
             asr_model = get_asr_model(args.model)
             pred_text = asr_model.transcribe([args.audio_data])[0]
             normalized_text, cer = normalizer.select_best_match(
-                normalized_texts, pred_text, args.verbose, args.remove_punct
+                normalized_texts=normalized_texts,
+                pred_text=pred_text,
+                input_text=args.text,
+                verbose=args.verbose,
+                remove_punct=args.remove_punct,
             )
-            print(f'Transcript: {pred_text}')
-            print(f'Normalized: {normalized_text}')
+            print(f"Transcript: {pred_text}")
+            print(f"Normalized: {normalized_text}")
         else:
-            print('Normalization options:')
+            print("Normalization options:")
             for norm_text in normalized_texts:
                 print(norm_text)
     elif not os.path.exists(args.audio_data):
-        raise ValueError(f'{args.audio_data} not found.')
+        raise ValueError(f"{args.audio_data} not found.")
     elif args.audio_data.endswith('.json'):
         normalize_manifest(args)
     else:
