@@ -111,6 +111,23 @@ def test_data_dir():
     return test_data_dir_
 
 
+def extract_data_from_tar(test_dir, test_data_archive, url=None):
+    # Remove .data folder.
+    if exists(test_dir):
+        rmtree(test_dir)
+    # Create one .data folder.
+    mkdir(test_dir)
+    # Download (if required)
+    if url is not None:
+        urllib.request.urlretrieve(url, test_data_archive)
+
+    # Extract tar
+    print("Extracting the `{}` test archive, please wait...".format(test_data_archive))
+    tar = tarfile.open(test_data_archive)
+    tar.extractall(path=test_dir)
+    tar.close()
+
+
 def pytest_configure(config):
     """
     Initial configuration of conftest.
@@ -141,51 +158,48 @@ def pytest_configure(config):
                     __TEST_DATA_FILENAME, test_data_local_size, test_dir
                 )
             )
-            return
 
     # Get size of remote test_data archive.
-    try:
-        url = __TEST_DATA_URL + __TEST_DATA_FILENAME
-        u = urllib.request.urlopen(url)
-    except:
-        # Couldn't access remote archive.
-        if test_data_local_size == -1:
-            pytest.exit("Test data not present in the system and cannot access the '{}' URL".format(url))
-        else:
+    if not config.option.use_local_test_data:
+        try:
+            url = __TEST_DATA_URL + __TEST_DATA_FILENAME
+            u = urllib.request.urlopen(url)
+        except:
+            # Couldn't access remote archive.
+            if test_data_local_size == -1:
+                pytest.exit("Test data not present in the system and cannot access the '{}' URL".format(url))
+            else:
+                print(
+                    "Cannot access the '{}' URL, using the test data ({}B) found in the `{}` folder.".format(
+                        url, test_data_local_size, test_dir
+                    )
+                )
+                return
+
+        # Get metadata.
+        meta = u.info()
+        test_data_remote_size = int(meta["Content-Length"])
+
+        # Compare sizes.
+        if test_data_local_size != test_data_remote_size:
             print(
-                "Cannot access the '{}' URL, using the test data ({}B) found in the `{}` folder.".format(
-                    url, test_data_local_size, test_dir
+                "Downloading the `{}` test archive from `{}`, please wait...".format(
+                    __TEST_DATA_FILENAME, __TEST_DATA_URL
                 )
             )
-            return
 
-    # Get metadata.
-    meta = u.info()
-    test_data_remote_size = int(meta["Content-Length"])
+            extract_data_from_tar(test_dir, test_data_archive, url=url)
 
-    # Compare sizes.
-    if test_data_local_size != test_data_remote_size:
-        print(
-            "Downloading the `{}` test archive from `{}`, please wait...".format(__TEST_DATA_FILENAME, __TEST_DATA_URL)
-        )
-        # Remove .data folder.
-        if exists(test_dir):
-            rmtree(test_dir)
-        # Create one .data folder.
-        mkdir(test_dir)
-        # Download
-        urllib.request.urlretrieve(url, test_data_archive)
-        # Extract tar
-        print("Extracting the `{}` test archive, please wait...".format(test_data_archive))
-        tar = tarfile.open(test_data_archive)
-        tar.extractall(path=test_dir)
-        tar.close()
-    else:
-        print(
-            "A valid `{}` test archive ({}B) found in the `{}` folder.".format(
-                __TEST_DATA_FILENAME, test_data_local_size, test_dir
+        else:
+            print(
+                "A valid `{}` test archive ({}B) found in the `{}` folder.".format(
+                    __TEST_DATA_FILENAME, test_data_local_size, test_dir
+                )
             )
-        )
+
+    else:
+        # untar local test data
+        extract_data_from_tar(test_dir, test_data_archive)
 
     if config.option.relax_numba_compat is not None:
         from nemo.core.utils import numba_utils
