@@ -43,7 +43,7 @@ class TestDate:
     )
     normalizer_with_audio_en = (
         NormalizerWithAudio(input_case='cased', lang='en', cache_dir=CACHE_DIR, overwrite_cache=False)
-        if PYNINI_AVAILABLE
+        if PYNINI_AVAILABLE and CACHE_DIR
         else None
     )
 
@@ -56,8 +56,12 @@ class TestDate:
     def test_norm_uncased(self, test_input, expected):
         pred = self.normalizer_en.normalize(test_input, verbose=False)
         assert pred == expected
-        pred_non_deterministic = self.normalizer_with_audio_en.normalize(test_input, n_tagged=100)
-        assert expected in pred_non_deterministic
+
+        if self.normalizer_with_audio_en:
+            pred_non_deterministic = self.normalizer_with_audio_en.normalize(
+                test_input, punct_post_process=False, n_tagged=100
+            )
+            assert expected in pred_non_deterministic
 
     normalizer_uppercased = (
         Normalizer(input_case='cased', lang='en', cache_dir=CACHE_DIR, overwrite_cache=False)
@@ -75,5 +79,9 @@ class TestDate:
     def test_norm_cased(self, test_input, expected):
         pred = self.normalizer_uppercased.normalize(test_input, verbose=False)
         assert pred == expected
-        pred_non_deterministic = self.normalizer_with_audio_en.normalize(test_input, n_tagged=100)
-        assert expected in pred_non_deterministic
+
+        if self.normalizer_with_audio_en:
+            pred_non_deterministic = self.normalizer_with_audio_en.normalize(
+                test_input, punct_post_process=False, n_tagged=30
+            )
+            assert expected in pred_non_deterministic
