@@ -44,7 +44,7 @@ class TestWord:
     )
     normalizer_with_audio_en = (
         NormalizerWithAudio(input_case='cased', lang='en', cache_dir=CACHE_DIR, overwrite_cache=False)
-        if PYNINI_AVAILABLE
+        if PYNINI_AVAILABLE and CACHE_DIR
         else None
     )
 
@@ -56,8 +56,10 @@ class TestWord:
     @pytest.mark.unit
     def test_norm(self, test_input, expected):
         pred = self.normalizer_en.normalize(test_input, verbose=False)
-        assert pred == expected
-        pred_non_deterministic = self.normalizer_with_audio_en.normalize(
-            test_input, n_tagged=100, punct_post_process=False
-        )
-        assert expected in pred_non_deterministic
+        assert pred == expected, f"input: {test_input}"
+
+        if self.normalizer_with_audio_en:
+            pred_non_deterministic = self.normalizer_with_audio_en.normalize(
+                test_input, n_tagged=150, punct_post_process=False
+            )
+            assert expected in pred_non_deterministic, f"input: {test_input}"
